/***************************************************************************//**
 *   @file   AD7794.h
 *   @brief  Header file of AD7794 Driver.
 *   @author Bancisor Mihai
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 500
*******************************************************************************/
#ifndef __AD7794_H__
#define __AD7794_H__

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/******************************************************************************/
/* GPIO Definitions                                                           */
/******************************************************************************/

//#define ADI_PART_CS_PIN        		// Add code here
//#define ADI_PART_CS_PIN_OUT   		// Add code here
#define ADI_PART_CS_LOW        		HAL_GPIO_WritePin(AD7794_nCS_GPIO_Port, AD7794_nCS_Pin, GPIO_PIN_RESET)
#define ADI_PART_CS_HIGH       		HAL_GPIO_WritePin(AD7794_nCS_GPIO_Port, AD7794_nCS_Pin, GPIO_PIN_SET)
//#define GPIO1_PIN              		// Add code here
//#define GPIO1_STATE            		1// Add code here
#define AD7794_RDY_STATE       		(HAL_GPIO_ReadPin(SPI1_MISO_GPIO_Port, SPI1_MISO_Pin)==GPIO_PIN_SET)

/******************************************************************************/
/* Functions Prototypes                                                       */
/******************************************************************************/
/* Initializes the SPI communication peripheral. */
unsigned char SPI_Init(unsigned char lsbFirst,
                       unsigned long clockFreq,
                       unsigned char clockPol,
                       unsigned char clockPha);
/* Writes data to SPI. */
unsigned char SPI_Write(unsigned char* data,
                        unsigned char bytesNumber);
/* Reads data from SPI. */
unsigned char SPI_Read(unsigned char* data,
                       unsigned char bytesNumber);

/******************************************************************************/
/* AD7794                                                                   */
/******************************************************************************/

/* AD7794 GPIO */

/*AD7794 Registers*/
#define AD7794_REG_COMM		0 /* Communications Register(WO, 8-bit) */
#define AD7794_REG_STAT	    0 /* Status Register	    (RO, 8-bit) */
#define AD7794_REG_MODE	    1 /* Mode Register	     	(RW, 16-bit */
#define AD7794_REG_CONF	    2 /* Configuration Register (RW, 16-bit)*/
#define AD7794_REG_DATA	    3 /* Data Register	     	(RO, 16-/24-bit) */
#define AD7794_REG_ID	    4 /* ID Register	     	(RO, 8-bit) */
#define AD7794_REG_IO	    5 /* IO Register	     	(RO, 8-bit) */
#define AD7794_REG_OFFSET   6 /* Offset Register	    (RW, 24-bit */
#define AD7794_REG_FULLSALE	7 /* Full-Scale Register	(RW, 24-bit */

/*
 * Communication register
 * | CR7 | CR6 | CR5 | CR4 | CR3 | CR2 | CR1 | CR0 |
 *  !WEN   R/!W  RS2   RS1   RS0  CREAD   0     0
 *
 * - WEN   (write enable)  : 1 - ignore all bits in packet
 * - R/!W  (next operation): 0 - write, 1 - read
 * - RS2..RS0 (reg choice) :
 * 		(RS2RS1RS0)
 * 		000: CR - write / SR - read (8 bit)
 * 		001: Mode register  (16 bit)
 * 		010: Configuration register  (16 bit)
 * 		011: Data register (24 bit)
 * 		100: ID register - read only (8 bit)
 * 		101: IO register (8 bit)
 * 		110: Offset register (24 bit)
 * 		111: Full-Scale register(24 bit)
 * - CREAD (continious read): 1 - enable continious reading (put 01011100 to
 * 			CR & take DIN in zero state
 * 			to reset this mode write to CR 01011000 or make ADC reset
 * CR1&CR0 always should be 0!
 */

/* Communications Register Bit Designations (AD7794_REG_COMM) */
#define AD7794_COMM_WEN		(1 << 7) 			/* Write Enable */
#define AD7794_COMM_WRITE	(0 << 6) 			/* Write Operation */
#define AD7794_COMM_READ    (1 << 6) 			/* Read Operation */
#define AD7794_COMM_ADDR(x)	(((x) & 0x7) << 3)	/* Register Address */
#define AD7794_COMM_CREAD	(1 << 2) 			/* Continuous Read of Data Register */

/*
 * Status register
 * | SR7 | SR6 | SR5 | SR4 | SR3 | SR2 | SR1 | SR0 |
 *  !RDY  ERR   NOXREF  0     1    CH2   CH1   CH0
 *
 * - !RDY (data ready)      : ==0 when data is ready for reading
 * - ERR (ADC error flag)   : ==1 in case of some error  (cleared by hardware in ADC starting)
 * - NOXREF (no ref voltage): ==1 in cese of REFIN1/REFIN2 less than threshold
 * SR4&SR3 are always 01
 * - CH2..CH0 (channel number): number of current channel
 */
 
/* Status Register Bit Designations (AD7794_REG_STAT) */
#define AD7794_STAT_RDY		(1 << 7) /* Ready */
#define AD7794_STAT_ERR		(1 << 6) /* Error (Overrange, Underrange) */
#define AD7794_STAT_NOXREF	(1 << 5) /* no ref voltage */
#define AD7794_STAT_CH3		(1 << 2) /* Channel 3 */
#define AD7794_STAT_CH2		(1 << 1) /* Channel 2 */
#define AD7794_STAT_CH1		(1 << 0) /* Channel 1 */

/*
 * Mode register (16 bit)
 * | MR15 | MR14 | MR13 | MR12 | MR11 | MR10 | MR9 | MR8 | MR7 | MR6 | MR5 | MR4 | MR3 | MR2 | MR1 | MR0 |
 *   MD2    MD1    MD0    PSW    0      0     AMP-CM  0    CLK1  CLK0   0 CHOP-DIS FS3   FS2   FS1   FS0
 *
 * - MD2..MD0 (mode)    : mode selection
 * 						000 - continuous mode
 * 						001 - single mode
 * 						010 - idle mode
 * 						011 - power-down mode
 * 						100 - internal zero-scale calibration
 * 						101 - internal full-scale calibration
 * 						110 - system zero-scale calibration
 * 						111 - system full-scale calibration
 * - PSW (PSW mode)     : ==0 to disconnect PSW and ground, ==1 to connect (!!! limit current is 30mA !!!)
 * - AMP-CM (amplify)   : ==0 to wide amplification (badly disables synphase signal)
 * - CLK1, CLK0 (CLCsrc): CLC source
 * 							00 - inner 64kHz, CLK out is off
 * 							01 - inner 64kHz, CLK out is on
 * 							10 - outer 64kHz connected to CLK in
 * 							11 - outer (freq/2) connected to CLK in
 * CHOP-DIS (syn dis)   : used to disable synphase signal, sets together with AMP-CM 1...chop disabled 0...chop enabled
 * FS3..FS0 (fltr per.) : period of filter update rate
 */

/* Mode Register Bit Designations (AD7794_REG_MODE) */
#define AD7794_MODE_SEL(x)		(((x) & 0x7) << 13)	/* Operation Mode Select */
#define AD7794_MODE_PSW			(1 << 12)
#define AD7794_MODE_CLKSRC(x)	(((x) & 0x3) << 6) 	/* ADC Clock Source Select */
#define AD7794_MODE_CHOP_DIS	(1 << 4)			
#define AD7794_MODE_RATE(x)		((x) & 0xF) 		/* Filter Update Rate Select */

/* AD7794_MODE_SEL(x) options */
#define AD7794_MODE_CONT		 0 /* Continuous Conversion Mode */
#define AD7794_MODE_SINGLE		 1 /* Single Conversion Mode */
#define AD7794_MODE_IDLE		 2 /* Idle Mode */
#define AD7794_MODE_PWRDN		 3 /* Power-Down Mode */
#define AD7794_MODE_CAL_INT_ZERO 4 /* Internal Zero-Scale Calibration */
#define AD7794_MODE_CAL_INT_FULL 5 /* Internal Full-Scale Calibration */
#define AD7794_MODE_CAL_SYS_ZERO 6 /* System Zero-Scale Calibration */
#define AD7794_MODE_CAL_SYS_FULL 7 /* System Full-Scale Calibration */

/* AD7794_MODE_CLKSRC(x) options */
#define AD7794_CLK_INT		0 /* Internal 64 kHz Clk not available at the CLK pin */
#define AD7794_CLK_INT_CO	1 /* Internal 64 kHz Clk available at the CLK pin */
#define AD7794_CLK_EXT		2 /* External 64 kHz Clock */
#define AD7794_CLK_EXT_DIV2	3 /* External Clock divided by 2 */

/*
 * Configuration register
 * | CR15 | CR14 | CR13 | CR12 | CR11 | CR10 | CR9 | CR8 | CR7 | CR6 | CR5 | CR4 | CR3 | CR2 | CR1 | CR0 |
 *    VBIAS1/0      BO    U/!B   BOOST   G2    G1    G0    REFSEL1/0 REF_DET BUF   CH3   CH2   CH1   CH0
 *
 * - VBIAS1/0 (enable bias voltage generator):
 * 							00 - VBIAS disabled
 * 							01 - connect to AIN1
 * 							10 - connect to AIN2
 * 							11 - connect to AIN3
 * - BO (burnout current)  : enable 100nA current source (buffer or in-amp should be active)
 * - U/!B (uni/bipolar)    : ==1 for unipolar coding (full-scale from 0x000000 to 0xffffff)
 * 						     ==0 for bipolar coding (0x800000 is zero, > - positive)
 * - BOOST (increase curnt): ==1 to increase inner VREF current consuming to reduce power-up time
 * - G2..G0 (gain)         : select ADC ampl. gain, GAIN = 1<<G, input ranges are
 * 			2.5V, 1.25V, 625mV, 312.5mV, 156.25mV, 78.125mV, 39.06mV, 19.53mV
 * - REFSEL1/0 (ref.select): reference voltage
 * 							00 - external between REFIN1(+) and REFIN1(-)
 * 							01 - external between REFIN2(+) and REFIN2(-)
 * 							10 - internal 1.17V
 * 							11 - reserved
 * - REF_DET (detect ref)  : enable reference detection by set/reset bit NOXREF
 * - BUF (buffered mode)   : configure ADC for buffered (1) mode  (==1 automatically for gain > 2)
 * - CH3..0 (chnnl select) : select channel (reserved are non-shown)
 * 							     [CHANNEL] - [Calibration pair]
 * 							0000 - AIN1 - 0
 * 							0001 - AIN2 - 1
 * 							0010 - AIN3 - 2
 * 							0011 - AIN4 - 3
 * 							0100 - AIN5 - 3
 * 							0101 - AIN6 - 3
 * 							0110 - Temp sensor  - int. 1.17VREF + gain=1
 * 							0111 - ADVV monitor - int. 1.17VREF + gain=1/6
 * 							1000 - AIN1(-)/AIN1(-) - 0
 */

/* Configuration Register Bit Designations (AD7794_REG_CONF) */
#define AD7794_CONF_VBIAS(x)  (((x) & 0x3) << 14) 	/* Bias Voltage Generator Enable */
#define AD7794_CONF_BO_EN	  (1 << 13) 			/* Burnout Current Enable */
#define AD7794_CONF_UNIPOLAR  (1 << 12) 			/* Unipolar/Bipolar Enable */
#define AD7794_CONF_BOOST	  (1 << 11) 			/* Boost Enable */
#define AD7794_CONF_GAIN(x)	  (((x) & 0x7) << 8) 	/* Gain Select */
#define AD7794_CONF_REFSEL(x) (((x) & 0x3) << 6) 	/* INT/EXT Reference Select */
#define AD7794_CONF_REF_DET   (1 << 5)				/* enable reference detection */
#define AD7794_CONF_BUF		  (1 << 4) 				/* Buffered Mode Enable */
#define AD7794_CONF_CHAN(x)	  ((x) & 0x15) 			/* Channel select */

/* AD7794_CONF_GAIN(x) options */
#define AD7794_GAIN_1       0
#define AD7794_GAIN_2       1
#define AD7794_GAIN_4       2
#define AD7794_GAIN_8       3
#define AD7794_GAIN_16      4
#define AD7794_GAIN_32      5
#define AD7794_GAIN_64      6
#define AD7794_GAIN_128     7

/* AD7794_CONF_REFSEL(x) options */
#define AD7794_REFSEL_INT    2	/* Internal Reference Selected. */
#define AD7794_REFSEL_EXT2   1	/* External Reference Applied between REFIN2(+) and REFIN2(). */
#define AD7794_REFSEL_EXT1   0	/* External Reference Applied between REFIN1(+) and REFIN1(). */

/* AD7794_CONF_CHAN(x) options */
#define AD7794_CH_AIN1P_AIN1M	0 /* AIN1(+) - AIN1(-) */
#define AD7794_CH_AIN2P_AIN2M	1 /* AIN2(+) - AIN2(-) */
#define AD7794_CH_AIN3P_AIN3M	2 /* AIN3(+) - AIN3(-) */
#define AD7794_CH_AIN1M_AIN1M	3 /* AIN1(-) - AIN1(-) */
#define AD7794_CH_TEMP			6 /* Temp Sensor */
#define AD7794_CH_AVDD_MONITOR	7 /* AVDD Monitor */


/*
 * Data register stores the conversion result: 24bits
 * on completion reading !RDY is set
 */


/*
 * ID register is read-only register storing identification number of ADC: 8bits
 */

/* ID Register Bit Designations (AD7794_REG_ID) */
#define AD7794_ID			0xF
#define AD7794_ID_MASK		0xF

/*
 * IO register
 * | IO7 | IO6 | IO5 | IO4 | IO3 | IO2 | IO1 | IO0 |
 *    0   IOEN     IODAT      IEXCDIR     IEXCEN
 *
 * - IOEN (digital out)   : ==1 to configure AIN6 as digital output pins P1/P2
 * - IODAT (P2/P1 data)   : when IOEN==1 this is a data for digital ports
 * - IEXCDIR (cur.src dir): direction of currens sources
 * 						00 - IEXC1 connected to IOUT1, IEXC2 connected to IOUT2
 * 						01 - IEXC1 connected to IOUT2, IEXC2 connected to IOUT1
 * 						10 - both sources connected to IOUT1 (only for 10/210uA)
 * 						11 - both sources connected to IOUT2 (-//-)
 * - IEXCEN (en. cur.src) : enable/disable current sources
 * 						00 - currents are disabled
 * 						01 - 10uA
 * 						10 - 210uA
 * 						11 - 1mA
 */

/* IO (Excitation Current Sources) Register Bit Designations (AD7794_REG_IO) */
#define AD7794_IEXCDIR(x)	(((x) & 0x3) << 2)
#define AD7794_IEXCEN(x)	(((x) & 0x3) << 0)

/* AD7794_IEXCDIR(x) options*/
#define AD7794_DIR_IEXC1_IOUT1_IEXC2_IOUT2	0  /* IEXC1 connect to IOUT1, IEXC2 connect to IOUT2 */
#define AD7794_DIR_IEXC1_IOUT2_IEXC2_IOUT1	1  /* IEXC1 connect to IOUT2, IEXC2 connect to IOUT1 */
#define AD7794_DIR_IEXC1_IEXC2_IOUT1		2  /* Both current sources IEXC1,2 connect to IOUT1  */
#define AD7794_DIR_IEXC1_IEXC2_IOUT2		3  /* Both current sources IEXC1,2 connect to IOUT2 */

/* AD7794_IEXCEN(x) options*/
#define AD7794_EN_IXCEN_10uA				1  /* Excitation Current 10uA */
#define AD7794_EN_IXCEN_210uA				2  /* Excitation Current 210uA */
#define AD7794_EN_IXCEN_1mA					3  /* Excitation Current 1mA */


/*
 * Offset register holds the offset calibration coefficient (R/W, 24bit)
 * Each calibration channel has its own offset registry
 * Value automatically overwtitten on calibration
 * To directly write into that registry ADC should be placed in power-down or idle mode
 */

/*
 * Full-scale register (R/W, 24 bit)
 * It holds the full-scale calibration coefficients fir ADC
 * Each calibration channel has its own full-scale registry
 * To directly write into that registry ADC should be placed in power-down or idle mode
 */

/******************************************************************************/
/* Functions Prototypes                                                       */
/******************************************************************************/

/* Initialize AD7794 and check if the device is present*/
unsigned char AD7794_Init(SPI_HandleTypeDef *hspimain);

/* Sends 32 consecutive 1's on SPI in order to reset the part. */
void AD7794_Reset(void);

/* Reads the value of the selected register. */
unsigned long AD7794_GetRegisterValue(unsigned char regAddress, 
									  unsigned char size,
                                      unsigned char modifyCS);

/* Writes a value to the register. */
void AD7794_SetRegisterValue(unsigned char regAddress,
							 unsigned long regValue, 
							 unsigned char size,
                             unsigned char modifyCS);

/* Waits for RDY pin to go low. */
void AD7794_WaitRdyGoLow(void);

/* Sets the operating mode of AD7794. */
void AD7794_SetMode(unsigned long mode);  

void AD7794_SetRate(unsigned long rate);

/* Selects the channel of AD7794. */
void AD7794_SetChannel(unsigned long channel);

/* Sets the gain of the In-Amp. */
void AD7794_SetGain(unsigned long gain);

/* Sets the reference source for the ADC. */
void AD7794_SetIntReference(unsigned char type);

/* Performs the given calibration to the specified channel. */
void AD7794_Calibrate(unsigned char mode, unsigned char channel);

/* Returns the result of a single conversion. */
unsigned long AD7794_SingleConversion(void);

unsigned long AD7794_ReadConversion(void);

/* Returns the average of several conversion results. */
unsigned long AD7794_ContinuousReadAvg(unsigned char sampleNumber);

#endif	// _AD7794_H_
