#include "main.h"
#include "AM2302.h"



////////////////////////////////////////////////////////////////////////////
// PRIVATE Variables
//
uint8_t data[6];
uint16_t _pin;
GPIO_TypeDef *_port;
uint8_t _type, _count;
unsigned long _lastreadtime;
char firstreading;
	
int DHTInit(GPIO_TypeDef *port, uint16_t pin, unsigned char type, unsigned char count)
{
  _pin = pin;
	_port = port;
  _type = type;
  _count = count;
  firstreading = true;
	
	HAL_GPIO_WritePin(_port, _pin, GPIO_PIN_SET);
  _lastreadtime = 0;
	
	return 0;
}

//boolean S == Scale.  True == Farenheit; False == Celcius
float DHTreadTemperature(char S) 
{
  float f;

  if (DHTread()) {
    switch (_type) {
    case DHT11:
      f = data[2];
      if(S)
      	f = DHTconvertCtoF(f);
      	
      return f;
    case DHT22:
    case DHT21:
      f = data[2] & 0x7F;
      f *= 256;
      f += data[3];
      f /= 10;
      if (data[2] & 0x80)
	f *= -1;
      if(S)
	f = DHTconvertCtoF(f);
      return f;
    }
  }
  return 1/0.0f;
}

float DHTreadHumidity(void) {
  float f;
  if (DHTread()) {
    switch (_type) {
    case DHT11:
      f = data[0];
      return f;
    case DHT22:
    case DHT21:
      f = data[0];
      f *= 256;
      f += data[1];
      f /= 10;
      return f;
    }
  }
  return 1/0.0f;
}

char DHTread(void) {
  GPIO_PinState laststate = GPIO_PIN_SET;
  uint8_t counter = 0;
  uint8_t j = 0, i;
  unsigned long currenttime;

  // Check if sensor was read less than two seconds ago and return early
  // to use last reading.
  currenttime = HAL_GetTick();
  if (currenttime < _lastreadtime) {
    // ie there was a rollover
    _lastreadtime = 0;
  }
  if (!firstreading && ((currenttime - _lastreadtime) < 2000)) {
    return true; // return last correct measurement
    //delay(2000 - (currenttime - _lastreadtime));
  }
  firstreading = false;
  /*
    Serial.print("Currtime: "); Serial.print(currenttime);
    Serial.print(" Lasttime: "); Serial.print(_lastreadtime);
  */
  _lastreadtime = HAL_GetTick();

  data[0] = data[1] = data[2] = data[3] = data[4] = 0;

  
  // pull the pin high and wait 250 milliseconds
  //digitalWrite(_pin, HIGH);
//	HAL_GPIO_WritePin(_port, _pin, GPIO_PIN_SET);
  //delay(250);
//	HAL_Delay(250);

  // now pull it low for ~20 milliseconds
//  pinMode(_pin, OUTPUT);
//  digitalWrite(_pin, LOW);
//  delay(20);
	HAL_GPIO_WritePin(_port, _pin, GPIO_PIN_RESET);
	HAL_Delay(20);

//	noInterrupts();
	__disable_irq();
//	digitalWrite(_pin, HIGH);
	HAL_GPIO_WritePin(_port, _pin, GPIO_PIN_SET);

	delayus(40);

  // read in timings
  for ( i=0; i< MAXTIMINGS; i++) {
    counter = 0;
    while (HAL_GPIO_ReadPin(_port, _pin) == laststate) {
      counter++;
      delayus(1);
      if (counter == 255) {
        break;
      }
    }
    laststate = HAL_GPIO_ReadPin(_port, _pin);

    if (counter == 255) break;

    // ignore first 3 transitions
    if ((i >= 4) && (i%2 == 0)) {
      // shove each bit into the storage bytes
      data[j/8] <<= 1;
      if (counter > _count)
        data[j/8] |= 1;
      j++;
    }
  }
//  interrupts();
  __enable_irq();
	
	HAL_GPIO_WritePin(_port, _pin, GPIO_PIN_SET);

  /*
  Serial.println(j, DEC);
  Serial.print(data[0], HEX); Serial.print(", ");
  Serial.print(data[1], HEX); Serial.print(", ");
  Serial.print(data[2], HEX); Serial.print(", ");
  Serial.print(data[3], HEX); Serial.print(", ");
  Serial.print(data[4], HEX); Serial.print(" =? ");
  Serial.println(data[0] + data[1] + data[2] + data[3], HEX);
  */

  // check we read 40 bits and that the checksum matches
  if ((j >= 40) && 
      (data[4] == ((data[0] + data[1] + data[2] + data[3]) & 0xFF)) ) {
    return true;
  }
  

  return false;

}

float DHTconvertCtoF(float c) {
	return c * 9 / 5 + 32;
}

