/**
  *
  * COPYRIGHT(c) 2014 STMicroelectronics
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/

// AN_312 FT800 header

#include "FT800.h"
#include "main.h"

#include "menu.h"

#include "rtc_stm32f1.h"

extern RTCTIME rtcTime;
extern structSettings Settings;

//Externe Variablen Menu
extern Active_Dialog_Function Active_Dialog;

char oldtag = 0, newtag = 0;		//newtag - wird gerade gedrckt

char Refresh = 0;

short xCursor,yCursor; //Diagram_Press

structMessageBox MessageBox;

SPI_HandleTypeDef *FT800_hspi;

// AN_312 FT800 pins
unsigned int ft800irqPin;																// Interrupt from FT800 to Arduino - not used here
unsigned int ft800pwrPin;																// PD_N from Arduino to FT800 - effectively FT800 reset
unsigned int ft800csPin;																// SPI chip select - defined separately since it's manipulated with GPIO calls

// AN_312 LCD display parameters
unsigned int lcdWidth;																	// Active width of LCD display
unsigned int lcdHeight;																	// Active height of LCD display
unsigned int lcdHcycle;																	// Total number of clocks per line
unsigned int lcdHoffset;																// Start of active line
unsigned int lcdHsync0;																	// Start of horizontal sync pulse
unsigned int lcdHsync1;																	// End of horizontal sync pulse
unsigned int lcdVcycle;																	// Total number of lines per screen
unsigned int lcdVoffset;																// Start of active screen
unsigned int lcdVsync0;																	// Start of vertical sync pulse
unsigned int lcdVsync1;																	// End of vertical sync pulse
unsigned char lcdPclk;																	// Pixel Clock
unsigned char lcdSwizzle;																// Define RGB output pins
unsigned char lcdPclkpol;																// Define active edge of PCLK

unsigned long ramDisplayList = RAM_DL;									// Set beginning of display list memory 
unsigned long ramCommandBuffer = RAM_CMD;								// Set beginning of graphics command memory

unsigned int cmdBufferRd = 0x0000;											// Used to navigate command ring buffer
unsigned int cmdBufferWr = 0x0000;											// Used to navigate command ring buffer
unsigned int cmdOffset = 0x0000;												// Used to navigate command rung buffer

long point_x[80];											// Define a default point x-location (1/16 anti-aliased)
long point_y[80];											// Define a default point y-location (1/16 anti-aliased)

unsigned long color;																		// Variable for chanign colors
unsigned char ft800Gpio;																// Used for FT800 GPIO register

structDiagram Diagram;

// AN_312 Specific function prototypes - See full descriptions below
unsigned int incCMDOffset(unsigned int, unsigned char);

int FT800_Init(SPI_HandleTypeDef *FT800_hspiInit)
{
	static unsigned char uctemp;
	
	FT800_hspi = FT800_hspiInit;

// LCD display parameters
#ifdef LCD_QVGA																					// QVGA display parameters
  lcdWidth   = 320;																			// Active width of LCD display
  lcdHeight  = 240;																			// Active height of LCD display
  lcdHcycle  = 408;																			// Total number of clocks per line
  lcdHoffset = 70;																			// Start of active line
  lcdHsync0  = 0;																				// Start of horizontal sync pulse
  lcdHsync1  = 10;																			// End of horizontal sync pulse
  lcdVcycle  = 263;																			// Total number of lines per screen
  lcdVoffset = 13;																			// Start of active screen
  lcdVsync0  = 0;																				// Start of vertical sync pulse
  lcdVsync1  = 2;																				// End of vertical sync pulse
  lcdPclk    = 8;																				// Pixel Clock
  lcdSwizzle = 2;																				// Define RGB output pins
  lcdPclkpol = 0;																				// Define active edge of PCLK
#endif

#ifdef LCD_WQVGA																				// WQVGA display parameters
  lcdWidth   = 480;																			// Active width of LCD display
  lcdHeight  = 272;																			// Active height of LCD display
  lcdHcycle  = 548;																			// Total number of clocks per line
  lcdHoffset = 43;																			// Start of active line
  lcdHsync0  = 0;																				// Start of horizontal sync pulse
  lcdHsync1  = 41;																			// End of horizontal sync pulse
  lcdVcycle  = 292;																			// Total number of lines per screen
  lcdVoffset = 12;																			// Start of active screen
  lcdVsync0  = 0;																				// Start of vertical sync pulse
  lcdVsync1  = 10;																			// End of vertical sync pulse
  lcdPclk    = 5;																				// Pixel Clock
  lcdSwizzle = 0;																				// Define RGB output pins
  lcdPclkpol = 1;																				// Define active edge of PCLK
#endif

	
	HAL_GPIO_WritePin(FT800_nPD_GPIO_Port, FT800_nPD_Pin, GPIO_PIN_SET); 			// Initial state of PD_N - high
	HAL_GPIO_WritePin(FT800_nCS_GPIO_Port, FT800_nCS_Pin, GPIO_PIN_SET);		// Initial state of SPI CS - high
	HAL_Delay(20);																				// Wait 20ms

	HAL_GPIO_WritePin(FT800_nPD_GPIO_Port, FT800_nPD_Pin, GPIO_PIN_RESET); // Reset FT800
	HAL_Delay(20);																				// Wait 20ms

	HAL_GPIO_WritePin(FT800_nPD_GPIO_Port, FT800_nPD_Pin, GPIO_PIN_SET); 	// FT800 is awake
	HAL_Delay(20);																				// Wait 20ms - required

	ft800cmdWrite(FT800_ACTIVE);													// Start FT800
  HAL_Delay(5);																					// Give some time to process

	ft800cmdWrite(FT800_CLKEXT);													// Set FT800 for external clock
  HAL_Delay(5);																					// Give some time to process

	ft800cmdWrite(FT800_CLK48M);													// Set FT800 for 48MHz PLL
  HAL_Delay(5);																					// Give some time to process

// Now FT800 can accept commands at up to 30MHz clock on SPI bus
  //   This application leaves the SPI bus at 8MHz
///////////////////////////////
//Getting stuck in the while loop here
//////////////////////////////////
	uctemp = ft800memRead8(REG_ID);

	if (uctemp != 0x7C)											// Read ID register - is it 0x7C?
  {
    return 0;	// If we don't get 0x7C, the ineface isn't working - halt with infinite loop
  }  
  
  ft800memWrite8(REG_PCLK, ZERO);												// Set PCLK to zero - don't clock the LCD until later
  ft800memWrite8(REG_PWM_DUTY, ZERO);										// Turn off backlight

// End of Wake-up FT800
	
//***************************************
// Initialize Display
  ft800memWrite16(REG_HSIZE,   lcdWidth);								// active display width
  ft800memWrite16(REG_HCYCLE,  lcdHcycle);							// total number of clocks per line, incl front/back porch
  ft800memWrite16(REG_HOFFSET, lcdHoffset);							// start of active line
  ft800memWrite16(REG_HSYNC0,  lcdHsync0);							// start of horizontal sync pulse
  ft800memWrite16(REG_HSYNC1,  lcdHsync1);							// end of horizontal sync pulse
  ft800memWrite16(REG_VSIZE,   lcdHeight);							// active display height
  ft800memWrite16(REG_VCYCLE,  lcdVcycle);							// total number of lines per screen, incl pre/post
  ft800memWrite16(REG_VOFFSET, lcdVoffset);							// start of active screen
  ft800memWrite16(REG_VSYNC0,  lcdVsync0);							// start of vertical sync pulse
  ft800memWrite16(REG_VSYNC1,  lcdVsync1);							// end of vertical sync pulse
  ft800memWrite8(REG_SWIZZLE,  lcdSwizzle);							// FT800 output to LCD - pin order
  ft800memWrite8(REG_PCLK_POL, lcdPclkpol);							// LCD data is clocked in on this PCLK edge
							// Don't set PCLK yet - wait for just after the first display list
// End of Initialize Display
//***************************************

//***************************************
// Configure Touch and Audio

	printf("Write FT800 Touch Settigns...\n");
	
	ft800memWrite32(REG_CTOUCH_TRANSFORM_A, Settings.ulSET_REG_CTOUCH_TRANSFORM_A);
	ft800memWrite32(REG_CTOUCH_TRANSFORM_B, Settings.ulSET_REG_CTOUCH_TRANSFORM_B);
	ft800memWrite32(REG_CTOUCH_TRANSFORM_C, Settings.ulSET_REG_CTOUCH_TRANSFORM_C);
	ft800memWrite32(REG_CTOUCH_TRANSFORM_D, Settings.ulSET_REG_CTOUCH_TRANSFORM_D);
	ft800memWrite32(REG_CTOUCH_TRANSFORM_E, Settings.ulSET_REG_CTOUCH_TRANSFORM_E);
	ft800memWrite32(REG_CTOUCH_TRANSFORM_F, Settings.ulSET_REG_CTOUCH_TRANSFORM_F);
	
  ft800memWrite8(REG_CTOUCH_MODE, TOUCHMODE_CONTINUOUS);									// Enable touch
	ft800memWrite8(REG_CTOUCH_EXTENDED, CTOUCHEXTENDED_COMPATIBILITY);		//Compatibility mode
//  ft800memWrite16(REG_TOUCH_RZTHRESH, 7200);						// Eliminate any false touches
  
  ft800memWrite8(REG_VOL_PB, ZERO);											// turn recorded audio volume down
  ft800memWrite8(REG_VOL_SOUND, ZERO);									// turn synthesizer volume down
  ft800memWrite16(REG_SOUND, 0x6000);										// set synthesizer to mute
  
// End of Configure Touch and Audio
//***************************************

//***************************************
// Write Initial Display List & Enable Display

  ramDisplayList = RAM_DL;															// start of Display List
  ft800memWrite32(ramDisplayList, DL_CLEAR_RGB); 				// Clear Color RGB   00000010 RRRRRRRR GGGGGGGG BBBBBBBB  (R/G/B = Colour values) default zero / black
  ramDisplayList += 4;																	// point to next location
  ft800memWrite32(ramDisplayList, (DL_CLEAR | CLR_COL | CLR_STN | CLR_TAG));	
																												// Clear 00100110 -------- -------- -----CST  (C/S/T define which parameters to clear)
  ramDisplayList += 4;																	// point to next location
  ft800memWrite32(ramDisplayList, DL_DISPLAY);					// DISPLAY command 00000000 00000000 00000000 00000000 (end of display list)

  ft800memWrite32(REG_DLSWAP, DLSWAP_FRAME);						// 00000000 00000000 00000000 000000SS  (SS bits define when render occurs)
																												// Nothing is being displayed yet... the pixel clock is still 0x00
  ramDisplayList = RAM_DL;															// Reset Display List pointer for next list
  
//  ft800Gpio = ft800memRead8(REG_GPIO);									// Read the FT800 GPIO register for a read/modify/write operation
//  ft800Gpio = ft800Gpio | 0x80;													// set bit 7 of FT800 GPIO register (DISP) - others are inputs
	
 // ft800memWrite8(REG_GPIO, ft800Gpio);									// Enable the DISP signal to the LCD panel
	
	//from demo
	ft800memWrite8(REG_GPIO_DIR, 0xff);
	ft800memWrite8(REG_GPIO, 0xff);
	
  ft800memWrite8(REG_PCLK, lcdPclk);										// Now start clocking data to the LCD panel

  ft800memWrite8(REG_PWM_DUTY, 128);									// Turn on backlight - ramp up slowly to full brighness

	
//End of Write Initial Display List & Enable Display
//***************************************	
//LOGO
	do
	{
		cmdBufferRd = ft800memRead16(REG_CMD_READ);					// Read the graphics processor read pointer
		cmdBufferRd &= 0x00000fff;
		cmdBufferWr = ft800memRead16(REG_CMD_WRITE); 				// Read the graphics processor write pointer
		cmdBufferWr &= 0x00000fff;
	}while (cmdBufferWr != cmdBufferRd);									// Wait until the two registers match
/*
	cmdOffset = cmdBufferWr;															// The new starting point the first location after the last command

	ft800memWrite32(RAM_CMD + cmdOffset, (CMD_DLSTART));// Start the display list
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer

	ft800memWrite32(RAM_CMD + cmdOffset, (CMD_LOGO));		
	cmdOffset = incCMDOffset(cmdOffset, 4);		

	ft800memWrite32(RAM_CMD + cmdOffset, (DL_DISPLAY));		// Instruct the graphics processor to show the list
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer

	ft800memWrite32(RAM_CMD + cmdOffset, (CMD_SWAP));			// Make this list active
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer

	ft800memWrite16(REG_CMD_WRITE, (cmdOffset));					// Update the ring buffer pointer so the graphics processor starts executing

	HAL_Delay(4000);
*/
	//FT800_Calibrate();
	
	return 1;
}


int	FT800_Calibrate(void)
{
	do
	{
		cmdBufferRd = ft800memRead16(REG_CMD_READ);					// Read the graphics processor read pointer
		cmdBufferWr = ft800memRead16(REG_CMD_WRITE); 				// Read the graphics processor write pointer
	}while (cmdBufferWr != cmdBufferRd);									// Wait until the two registers match

	cmdOffset = cmdBufferWr;															// The new starting point the first location after the last command
	
	ft800memWrite32(RAM_CMD + cmdOffset, (CMD_DLSTART));// Start the display list
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
	ft800memWrite32(RAM_CMD + cmdOffset, (DL_CLEAR_RGB | BLACK));// Set the default clear color to black
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
	ft800memWrite32(RAM_CMD + cmdOffset, (DL_CLEAR | CLR_COL | CLR_STN | CLR_TAG)); // Clear the screen - this and the previous prevent artifacts between lists
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer

	cmd_text(20,100,16,0,"Please tap on the dot   ");
	cmd_calibrate();
	
	ft800memWrite32(RAM_CMD + cmdOffset, (DL_DISPLAY));		// Instruct the graphics processor to show the list
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
	ft800memWrite32(RAM_CMD + cmdOffset, (CMD_SWAP));			// Make this list active
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
	ft800memWrite16(REG_CMD_WRITE, (cmdOffset));					// Update the ring buffer pointer so the graphics processor starts executing


	do
	{
		cmdBufferRd = ft800memRead16(REG_CMD_READ);					// Read the graphics processor read pointer
		cmdBufferWr = ft800memRead16(REG_CMD_WRITE); 				// Read the graphics processor write pointer
	}while (cmdBufferWr != cmdBufferRd);									// Wait until the two registers match
	
	return 0;
}

int cmd_calibrate(void)
{
	ft800memWrite32(RAM_CMD + cmdOffset, CMD_CALIBRATE); // Indicate to draw a point (dot)
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
	
	return 0;
}

char FT800_Read_Tag(void)
{
	char temp;
	
	newtag = ft800memRead8(REG_TOUCH_TAG);
	if (newtag == 0)
	{
		if(oldtag)
			Refresh = 1;
		temp = oldtag;
		oldtag = 0;
		return temp;
	}
	if (oldtag != newtag)
	{
		oldtag = newtag;
		Refresh = 1;
	}
	return 0;
}

int Draw_Display(char inputtag)
{

	// Wait for graphics processor to complete executing the current command list
	// This happens when REG_CMD_READ matches REG_CMD_WRITE, indicating that all commands
	// have been executed.  The next commands get executed when REG_CMD_WRITE is updated again...
	// then REG_CMD_READ again catches up to REG_CMD_WRITE
	// This is a 4Kbyte ring buffer, so keep pointers within the 4K roll-over
	do
	{
		cmdBufferRd = ft800memRead16(REG_CMD_READ);					// Read the graphics processor read pointer
		cmdBufferWr = ft800memRead16(REG_CMD_WRITE); 				// Read the graphics processor write pointer
	}while (cmdBufferWr != cmdBufferRd);									// Wait until the two registers match

	cmdOffset = cmdBufferWr;															// The new starting point the first location after the last command
	
	ft800memWrite32(RAM_CMD + cmdOffset, (CMD_DLSTART));// Start the display list
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
	ft800memWrite32(RAM_CMD + cmdOffset, (DL_CLEAR_RGB | BLACK));// Set the default clear color to black
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
	ft800memWrite32(RAM_CMD + cmdOffset, (DL_CLEAR | CLR_COL | CLR_STN | CLR_TAG)); // Clear the screen - this and the previous prevent artifacts between lists
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
	
 	Diagram_Draw(&Diagram);
 	
	static char s[60];
	
	//Beschriftung Solltemp Isttemp
// 	color_rgb(200,200,200);
// 	snprintf(s,15,"Solltemperatur");
// 	cmd_text(30, 20, 28, 0, s);
// 	snprintf(s,15,"Isttemperatur");
// 	cmd_text(190, 20, 28, 0, s);
	
	//Solltemp
// 	snprintf(s,15,"% 5.2fC",PID.w);
// 	color_a(255);
// 	color_rgb(255,255,120);
// 	cmd_text(160, 40, 31, OPT_RIGHTX, s);

//Weight
	snprintf(s,15,"% 6.3fkg",fweightcalibrated);
	color_a(255);
	color_rgb(255,255,120);
	cmd_text(190, 40, 31, OPT_RIGHTX, s);	

//fTemp1
	snprintf(s,15,"% 5.2fC",ftemp1);
	color_a(255);
	color_rgb(255,255,120);
	cmd_text(350, 40, 31, OPT_RIGHTX, s);


	//Temp
	color_rgb(200,200,200);
	snprintf(s,15,"Temp2 %5.2fC",ftemp2);
	cmd_text(10, 120, 28, 0, s);
	snprintf(s,15,"Temp3 %5.2fC",DHTTemp);
	cmd_text(10, 140, 28, 0, s);
	snprintf(s,15,"RH %5.2f%%",DHTHumidity);
	cmd_text(10, 160, 28, 0, s);

	//VBat
	snprintf(s,15,"VBat %6.3f V",fBatVoltage);
	cmd_text(220, 15, 28, 0, s);

	
	//PID
// 	snprintf(s,60,"w %5.2f en %5.3f P %5.3f I %5.3f D %5.3f un %5.3f\n", PID.w, PID.en, PID.P, PID.I, PID.D, PID.un);
// 	cmd_text(10, 180, 20, 0, s);
	
	//Date
	color_rgb(200,200,200);
	snprintf(s,15,"%02u:%02u:%02u",rtcTime.hour, rtcTime.min, rtcTime.sec);
	cmd_text(376,246,20,OPT_RIGHTX,s);
	snprintf(s,15,"%02u.%02u.%04u",rtcTime.mday, rtcTime.month, rtcTime.year);
	cmd_text(376,256,20,OPT_RIGHTX,s);

	Menu_Draw(newtag);
 	
 	if (Active_Dialog)
 		Active_Dialog(newtag, inputtag);
	
	if (MessageBox.Active)
		DrawMessageBox();
	
	
	ft800memWrite32(RAM_CMD + cmdOffset, (DL_DISPLAY));		// Instruct the graphics processor to show the list
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
	ft800memWrite32(RAM_CMD + cmdOffset, (CMD_SWAP));			// Make this list active
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
	ft800memWrite16(REG_CMD_WRITE, (cmdOffset));					// Update the ring buffer pointer so the graphics processor starts executing

	Refresh = 0;
	return 0;
}

int DrawMessageBox()
{
	color_a(200);
	color_rgb(40,40,40);
	begin(RECTS);
	line_width(100);
	vertex2ii(  15, 220, 0, 120);
	vertex2ii(320, 250, 0, 120);
	end();
	color_a(255);
	color_rgb(255,50,50);
	cmd_text(20, 220, 18, 0, MessageBox.pString);	
	
	return 0;
}

int InitDiagram(structDiagram *pDiagram, structData *pData1, structData *pData2)
{
	pDiagram->xPosition=0;
	pDiagram->yPosition=0;
	pDiagram->xPixel = 381;
	pDiagram->yPixel = 272;
	pDiagram->xValueStart=0;
	pDiagram->xValueEnd=DATASIZE;
	pDiagram->yValueStart=Settings.DiagramMinY;
	pDiagram->yValueEnd=Settings.DiagramMaxY;
	pDiagram->NoPoints=DATASIZE;
	pDiagram->pData1 = pData1;
	pDiagram->pData2 = pData2;
	
	CalculateDiagramConstants(pDiagram);
	
	return 0;
}

int CalculateDiagramConstants(structDiagram *pDiagram)
{
	pDiagram->kx = ( pDiagram->xPixel) / (pDiagram->xValueEnd - pDiagram->xValueStart);
	pDiagram->dx = pDiagram->xPosition - pDiagram->kx * pDiagram->xValueStart;
	
	pDiagram->ky = (-pDiagram->yPixel) / (pDiagram->yValueEnd - pDiagram->yValueStart);
	pDiagram->dy = pDiagram->yPosition - pDiagram->ky * pDiagram->yValueEnd;
	
	return 0;
}

int Diagram_Press(void)
{
	unsigned long temp;

	temp = ft800memRead32(REG_TOUCH_SCREEN_XY);
	xCursor=(short) (temp >> 16);
	yCursor=(short) (temp & 0x0000FFFFUL);
	return 0;
}

int Diagram_Draw(structDiagram *pDiagram)
{
	int i;
	float xtemp, ytemp;
	
	// Raster
	color_rgb(100,100,100);
	line_width(8);
	
	begin(LINES);
	
	for(i=1; i<10; i++)
	{
		//Vertikal
		vertex2ii((short)((float)(pDiagram->xPixel)/(float)10*(float)i) + pDiagram->xPosition, pDiagram->yPosition, 0, 0);
		vertex2ii((short)((float)(pDiagram->xPixel)/(float)10*(float)i) + pDiagram->xPosition, pDiagram->yPosition + pDiagram->yPixel, 0, 0);
	}

	for(i=1; i<8; i++)
	{
		//Horizontal
		vertex2ii(pDiagram->xPosition ,(short)((float)(pDiagram->yPixel) / (float)8 * (float)i) + pDiagram->yPosition, 0, 0);
		vertex2ii(pDiagram->xPosition+pDiagram->xPixel,(short)((float)(pDiagram->yPixel) / (float)8 * (float)i) + pDiagram->yPosition, 0, 0);
	}	
	end();
	
	//Dummy RECT fr tag Erkennung
	tag(20);
	color_mask(0,0,0,0);
	begin(RECTS);
	vertex2ii(pDiagram->xPosition, pDiagram->yPosition, 0, 0);
	vertex2ii(pDiagram->xPosition+pDiagram->xPixel, pDiagram->yPosition+pDiagram->yPixel, 0, 0);
	end();
	color_mask(1,1,1,1);
	
	//Data
	color_rgb(255,255,100);
	line_width(16);
	begin(LINE_STRIP);

	for(i=pDiagram->pData1->iNext; i<pDiagram->NoPoints; i++)
	{
		xtemp = 16.0 * ((pDiagram->kx * (i-pDiagram->pData1->iNext)) + pDiagram->dx);
		ytemp = 16.0 * ((pDiagram->ky * pDiagram->pData1->fArray[i]) + pDiagram->dy);
		
		if(xtemp > 8192.0) xtemp = 8192.0;
		if(xtemp < 0.0) xtemp = 0.0;
		
		if(ytemp > 8192.0) ytemp = 8192.0;
		if(ytemp < 0.0) ytemp = 0.0;
		
		vertex2f((short)(xtemp), (short) ytemp);
	}

	for(i=0; i<pDiagram->pData1->iNext; i++)
	{
		xtemp = 16.0 * ((pDiagram->kx * (i+pDiagram->NoPoints-pDiagram->pData1->iNext)) + pDiagram->dx);
		ytemp = 16.0 * ((pDiagram->ky * pDiagram->pData1->fArray[i]) + pDiagram->dy);
		
		if(xtemp > 8192.0) xtemp = 8192.0;
		if(xtemp < 0.0) xtemp = 0.0;
		
		if(ytemp > 8192.0) ytemp = 8192.0;
		if(ytemp < 0.0) ytemp = 0.0;
		
		vertex2f((short)(xtemp), (short) ytemp);
	}
	
	end();
/*
	color_rgb(100,255,100);
	begin(LINE_STRIP);
	
	for(i=pDiagram->pData2->iNext; i<pDiagram->NoPoints; i++)
	{
		xtemp = 16.0 * ((pDiagram->kx * (i-pDiagram->pData2->iNext)) + pDiagram->dx);
		ytemp = 16.0 * ((pDiagram->ky * pDiagram->pData2->fArray[i]) + pDiagram->dy);
		
		if(xtemp > 8192.0) xtemp = 8192.0;
		if(xtemp < 0.0) xtemp = 0.0;
		
		if(ytemp > 8192.0) ytemp = 8192.0;
		if(ytemp < 0.0) ytemp = 0.0;
		
		vertex2f((short)(xtemp), (short) ytemp);
	}

	for(i=0; i<pDiagram->pData2->iNext; i++)
	{
		xtemp = 16.0 * ((pDiagram->kx * (i+pDiagram->NoPoints-pDiagram->pData2->iNext)) + pDiagram->dx);
		ytemp = 16.0 * ((pDiagram->ky * pDiagram->pData2->fArray[i]) + pDiagram->dy);
		
		if(xtemp > 8192.0) xtemp = 8192.0;
		if(xtemp < 0.0) xtemp = 0.0;
		
		if(ytemp > 8192.0) ytemp = 8192.0;
		if(ytemp < 0.0) ytemp = 0.0;
		
		vertex2f((short)(xtemp), (short) ytemp);
	}	

	end();
*/
// 	//Sollwert
// 	color_rgb(100,100,255);
// 	begin(LINE_STRIP);
// 	xtemp = 16.0 * ((pDiagram->kx * 0 + pDiagram->dx));
// 	ytemp = 16.0 * ((pDiagram->ky * PID.w + pDiagram->dy));
// 	
// 	if(xtemp > 8192.0) xtemp = 8192.0;
// 	if(xtemp < 0.0) xtemp = 0.0;
// 	
// 	if(ytemp > 8192.0) ytemp = 8192.0;
// 	if(ytemp < 0.0) ytemp = 0.0;
// 	
// 	vertex2f((short)(xtemp), (short) ytemp);
// 	xtemp = 16.0 * ((pDiagram->kx * 400 + pDiagram->dx));
// 	ytemp = 16.0 * ((pDiagram->ky * PID.w + pDiagram->dy));
// 	
// 	if(xtemp > 8192.0) xtemp = 8192.0;
// 	if(xtemp < 0.0) xtemp = 0.0;
// 	
// 	if(ytemp > 8192.0) ytemp = 8192.0;
// 	if(ytemp < 0.0) ytemp = 0.0;
// 	
// 	vertex2f((short)(xtemp), (short) ytemp);
// 	
// 	end();

	//Cursor
	color_rgb(255,50,50);
	point_size(64);
	begin(FTPOINTS);
	vertex2ii(xCursor,yCursor,0,0);
	end();


	//Umrandung
	color_rgb(255,255,255);
	begin(LINE_STRIP);
	line_width(32);
	vertex2ii(pDiagram->xPosition, pDiagram->yPosition, 0, 120);
	vertex2ii(pDiagram->xPosition+pDiagram->xPixel, pDiagram->yPosition, 0, 120);
	vertex2ii(pDiagram->xPosition+pDiagram->xPixel, pDiagram->yPosition+pDiagram->yPixel, 0, 120);
	vertex2ii(pDiagram->xPosition, pDiagram->yPosition+pDiagram->yPixel, 0, 120);
	vertex2ii(pDiagram->xPosition, pDiagram->yPosition, 0, 120);	
	end();
/*	
	color_rgb(255,255,255);
	sprintf(buf, "%idBm",Refline);
	cmd_text(x+5,y+5,16,0,buf);								//Reference Line
*/
	char buf[20];
	snprintf(buf,20, "%3.2fkg/DIV",(pDiagram->yValueEnd-pDiagram->yValueStart)/8.0);
	cmd_text(pDiagram->xPosition + 5,pDiagram->yPosition+5,16,0,buf);	

	snprintf(buf,20, "%3.1fkg",(pDiagram->yValueStart));
	cmd_text(pDiagram->xPosition + 5,pDiagram->yPosition+pDiagram->yPixel-10,16,0,buf);	

	snprintf(buf,20, "%3.1fmin/DIV",(pDiagram->xValueEnd-pDiagram->xValueStart)/10.0*Settings.LogToThingSpeakInterval);
	cmd_text(pDiagram->xPosition+pDiagram->xPixel/2.0,pDiagram->yPosition+pDiagram->yPixel-10,16,OPT_CENTERX,buf);	

	
	return 0;
}

int MessageBoxText(const char* pString, char Active)
{
	if(Active		&& pString!=0)
	{
		MessageBox.pString = pString;
		MessageBox.Active = 1;
	}
	else
	{
		MessageBox.pString=0;
		MessageBox.Active = 0;
	}
	return MessageBox.Active;
}

void end (void)
{
	ft800memWrite32(RAM_CMD + cmdOffset, DL_END); // Indicate to draw a point (dot)
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
}

void tag (short s)
{
	ft800memWrite32(RAM_CMD + cmdOffset, (DL_TAG | s)); // Indicate to draw a point (dot)
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
}


void cmd_button(short x, short y, short w, short h, short font, short options, const char*s)
{
	int i=0;
	
	ft800memWrite32(RAM_CMD + cmdOffset, (CMD_BUTTON));
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
	ft800memWrite16(RAM_CMD + cmdOffset, (x));
	cmdOffset = incCMDOffset(cmdOffset, 2);								// Update the command pointer
	ft800memWrite16(RAM_CMD + cmdOffset, (y));
	cmdOffset = incCMDOffset(cmdOffset, 2);								// Update the command pointer
	ft800memWrite16(RAM_CMD + cmdOffset, (w));
	cmdOffset = incCMDOffset(cmdOffset, 2);								// Update the command pointer
	ft800memWrite16(RAM_CMD + cmdOffset, (h));
	cmdOffset = incCMDOffset(cmdOffset, 2);								// Update the command pointer
	ft800memWrite16(RAM_CMD + cmdOffset, (font));
	cmdOffset = incCMDOffset(cmdOffset, 2);								// Update the command pointer
	ft800memWrite16(RAM_CMD + cmdOffset, (options));
	cmdOffset = incCMDOffset(cmdOffset, 2);								// Update the command pointer
	
	i=-1;
	do
	{
		i++;
		ft800memWrite8(RAM_CMD + cmdOffset, s[i]);
		cmdOffset = incCMDOffset(cmdOffset, 1);								// Update the command pointer
	}while( s[i] != 0);
	
	while (cmdOffset%4)		//Restliche Bytes mit 0 fllen
	{
		ft800memWrite8(RAM_CMD + cmdOffset, 0);
		cmdOffset = incCMDOffset(cmdOffset, 1);								// Update the command pointer
	}
	
}

void cmd_keys(short x, short y, short w, short h, short font, short options, const char*s)
{
	int i=0;
	
	ft800memWrite32(RAM_CMD + cmdOffset, (CMD_KEYS));
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
	ft800memWrite16(RAM_CMD + cmdOffset, (x));
	cmdOffset = incCMDOffset(cmdOffset, 2);								// Update the command pointer
	ft800memWrite16(RAM_CMD + cmdOffset, (y));
	cmdOffset = incCMDOffset(cmdOffset, 2);								// Update the command pointer
	ft800memWrite16(RAM_CMD + cmdOffset, (w));
	cmdOffset = incCMDOffset(cmdOffset, 2);								// Update the command pointer
	ft800memWrite16(RAM_CMD + cmdOffset, (h));
	cmdOffset = incCMDOffset(cmdOffset, 2);								// Update the command pointer
	ft800memWrite16(RAM_CMD + cmdOffset, (font));
	cmdOffset = incCMDOffset(cmdOffset, 2);								// Update the command pointer
	ft800memWrite16(RAM_CMD + cmdOffset, (options));
	cmdOffset = incCMDOffset(cmdOffset, 2);								// Update the command pointer
	
	i=-1;
	do
	{
		i++;
		ft800memWrite8(RAM_CMD + cmdOffset, s[i]);
		cmdOffset = incCMDOffset(cmdOffset, 1);								// Update the command pointer
	}while( s[i] != 0);
	
	while (cmdOffset%4)		//Restliche Bytes mit 0 fllen
	{
		ft800memWrite8(RAM_CMD + cmdOffset, 0);
		cmdOffset = incCMDOffset(cmdOffset, 1);								// Update the command pointer
	}
	
}

void line_width (short width)
{
	ft800memWrite32(RAM_CMD + cmdOffset, (DL_LINE_WIDTH | width)); // Indicate to draw a point (dot)
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
}
void point_size (short size)
{
	ft800memWrite32(RAM_CMD + cmdOffset, (DL_POINT_SIZE |  size));// Select the size of the dot to draw
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
}

void vertex2ii(short x, short y, short handle, short cell)
{
	ft800memWrite32(RAM_CMD + cmdOffset, (DL_VERTEX2II | (x << 21) | (y << 12) | (handle << 7) | cell ) );  // Set the point center location
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
}

void vertex2f(short x, short y)
{
	ft800memWrite32(RAM_CMD + cmdOffset, (DL_VERTEX2F | (x << 15) | y) );  // Set the point center location
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
}

void cmd_fgcolor(short r, short g, short b)
{
	ft800memWrite32(RAM_CMD + cmdOffset, (CMD_FGCOLOR));	// Set the color of the following item(s) - toggle red/white from above
 	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
	
	ft800memWrite32(RAM_CMD + cmdOffset, RGB( r,g,b ) );	// Set the color of the following item(s) - toggle red/white from above
 	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
}


void begin (char mode)
{
	ft800memWrite32(RAM_CMD + cmdOffset, (DL_BEGIN | mode)); // Indicate to draw a point (dot)
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
}

void color_rgb(short r, short g, short b)
{
	ft800memWrite32(RAM_CMD + cmdOffset, (DL_COLOR_RGB | RGB( r,g,b )));
 																												// Set the color of the following item(s) - toggle red/white from above
 	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
}

void color_mask(char r, char g, char b, char a)
{
	ft800memWrite32(RAM_CMD + cmdOffset, (DL_COLOR_MASK | (r<<3) | (g<<2) | (b<<1) | a ));// Set the color of the following item(s) - toggle red/white from above
 	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
}


void color_a(short a)
{
	ft800memWrite32(RAM_CMD + cmdOffset, (DL_COLOR_A | a ) );
 																												// Set the color of the following item(s) - toggle red/white from above
 	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
}


void cmd_text(short x, short y, short font, unsigned short options, const char*s)
{
	int i=0;
	
	ft800memWrite32(RAM_CMD + cmdOffset, (CMD_TEXT));
	cmdOffset = incCMDOffset(cmdOffset, 4);								// Update the command pointer
	ft800memWrite16(RAM_CMD + cmdOffset, (x));
	cmdOffset = incCMDOffset(cmdOffset, 2);								// Update the command pointer
	ft800memWrite16(RAM_CMD + cmdOffset, (y));
	cmdOffset = incCMDOffset(cmdOffset, 2);								// Update the command pointer
	ft800memWrite16(RAM_CMD + cmdOffset, (font));
	cmdOffset = incCMDOffset(cmdOffset, 2);								// Update the command pointer
	ft800memWrite16(RAM_CMD + cmdOffset, (options));
	cmdOffset = incCMDOffset(cmdOffset, 2);								// Update the command pointer
	
	i=-1;
	do
	{
		i++;
		ft800memWrite8(RAM_CMD + cmdOffset, s[i]);
		cmdOffset = incCMDOffset(cmdOffset, 1);								// Update the command pointer
	}while( s[i] != 0);
	
	while (cmdOffset%4)		//Restliche Bytes mit 0 fllen
	{
		ft800memWrite8(RAM_CMD + cmdOffset, 0);
		cmdOffset = incCMDOffset(cmdOffset, 1);								// Update the command pointer
	}
}

/******************************************************************************
 * Function:        void ft800cmdWrite(ftCommand)
 * PreCondition:    None
 * Input:           ftCommand
 * Output:          None
 * Side Effects:    None
 * Overview:        Sends FT800 command
 * Note:            None
 *****************************************************************************/
void ft800cmdWrite(unsigned char ftCommand)
{
	unsigned char cZero = 0x00;														// Filler value for command
	HAL_GPIO_WritePin(FT800_nCS_GPIO_Port, FT800_nCS_Pin, GPIO_PIN_RESET);	// Set chip select low

 	HAL_SPI_Transmit(FT800_hspi, &ftCommand, 1, 100);						// Send command
 	HAL_SPI_Transmit(FT800_hspi, &cZero, 1, 100);								// Send first filler byte
 	HAL_SPI_Transmit(FT800_hspi, &cZero, 1, 100);								// Send second filler byte

	//while(SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_BSY) == SET);		//Warten auf letztes Bit
	
	HAL_GPIO_WritePin(FT800_nCS_GPIO_Port, FT800_nCS_Pin, GPIO_PIN_SET);		// Set chip select high
}

/******************************************************************************
 * Function:        void ft800memWritexx(ftAddress, ftDataxx, ftLength)
 * PreCondition:    None
 * Input:           ftAddress = FT800 memory space address
 *                  ftDataxx = a byte, int or long to send
 * Output:          None
 * Side Effects:    None
 * Overview:        Writes FT800 internal address space
 * Note:            "xx" is one of 8, 16 or 32
 *****************************************************************************/
void ft800memWrite8(unsigned long ftAddress, unsigned char ftData8)
{
	int i,j;
	unsigned char cTempAddr[3];														// FT800 Memory Address

	cTempAddr[2] = (char) (ftAddress >> 16) | MEM_WRITE;	// Compose the command and address to send
	cTempAddr[1] = (char) (ftAddress >> 8);								// middle byte
	cTempAddr[0] = (char) (ftAddress);										// low byte
	
	HAL_GPIO_WritePin(FT800_nCS_GPIO_Port, FT800_nCS_Pin, GPIO_PIN_RESET);	// Set chip select low
  
  for (i = 2; i >= 0; i--)
	{
		HAL_SPI_Transmit(FT800_hspi, &cTempAddr[i], 1, 100); 			// Send Memory Write plus high address byte
	}
	
	for (j = 0; j < sizeof(ftData8); j++)
	{
		HAL_SPI_Transmit(FT800_hspi, &ftData8, 1, 100);						// Send data byte	
	}
//	while(SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_BSY) == SET);		//Warten auf letztes Bit
	HAL_GPIO_WritePin(FT800_nCS_GPIO_Port, FT800_nCS_Pin, GPIO_PIN_SET);		// Set chip select high

}

void ft800memWrite16(unsigned long ftAddress, unsigned int ftData16)
{
	int i,j;
	unsigned char cTempAddr[3];														// FT800 Memory Address
	unsigned char cTempData[2];														// 16-bit data to write

	cTempAddr[2] = (char) (ftAddress >> 16) | MEM_WRITE;	// Compose the command and address to send
	cTempAddr[1] = (char) (ftAddress >> 8);								// middle byte
	cTempAddr[0] = (char) (ftAddress);										// low byte
	
	cTempData[1] = (char) (ftData16 >> 8);								// Compose data to be sent - high byte
	cTempData[0] = (char) (ftData16);											// low byte
	
	HAL_GPIO_WritePin(FT800_nCS_GPIO_Port, FT800_nCS_Pin, GPIO_PIN_RESET);	// Set chip select low
  
  for (i = 2; i >= 0; i--)
	{
		HAL_SPI_Transmit(FT800_hspi, &cTempAddr[i], 1, 100); 			// Send Memory Write plus high address byte
	}

	for (j = 0; j < sizeof(cTempData); j++)						// Start with least significant byte
	{
		HAL_SPI_Transmit(FT800_hspi, &cTempData[j], 1, 100);			// Send data byte	
	}
//	while(SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_BSY) == SET);		//Warten auf letztes Bit
	HAL_GPIO_WritePin(FT800_nCS_GPIO_Port, FT800_nCS_Pin, GPIO_PIN_SET);		// Set chip select high

}

void ft800memWrite32(unsigned long ftAddress, unsigned long ftData32)
{
	int i,j;
	unsigned char cTempAddr[3];														// FT800 Memory Address
	unsigned char cTempData[4];														// 32-bit data to write

	cTempAddr[2] = (char) (ftAddress >> 16) | MEM_WRITE;	// Compose the command and address to send
	cTempAddr[1] = (char) (ftAddress >> 8);								// middle byte
	cTempAddr[0] = (char) (ftAddress);										// low byte
	
	cTempData[3] = (char) (ftData32 >> 24);								// Compose data to be sent - high byte
	cTempData[2] = (char) (ftData32 >> 16);								// 
	cTempData[1] = (char) (ftData32 >> 8);								//
	cTempData[0] = (char) (ftData32);											// low byte
	
	HAL_GPIO_WritePin(FT800_nCS_GPIO_Port, FT800_nCS_Pin, GPIO_PIN_RESET);	// Set chip select low
  
  for (i = 2; i >= 0; i--)
	{
		HAL_SPI_Transmit(FT800_hspi, &cTempAddr[i], 1, 100); 			// Send Memory Write plus high address byte
	}
	
	for (j = 0; j < sizeof(cTempData); j++)						// Start with least significant byte
	{
		HAL_SPI_Transmit(FT800_hspi, &cTempData[j], 1, 100);			// Send SPI byte
	}
//	while(SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_BSY) == SET);		//Warten auf letztes Bit
	HAL_GPIO_WritePin(FT800_nCS_GPIO_Port, FT800_nCS_Pin, GPIO_PIN_SET);		// Set chip select high

}
/******************************************************************************
 * Function:        unsigned char ft800memReadxx(ftAddress, ftLength)
 * PreCondition:    None
 * Input:           ftAddress = FT800 memory space address
 * Output:          ftDataxx (byte, int or long)
 * Side Effects:    None
 * Overview:        Reads FT800 internal address space
 * Note:            "xx" is one of 8, 16 or 32
 *****************************************************************************/
unsigned char ft800memRead8(unsigned long ftAddress)
{
	int i,j;
  unsigned char ftData8 = ZERO;													// Place-holder for 8-bits being read
	unsigned char cTempAddr[3];														// FT800 Memory Address
	unsigned char cZeroFill = ZERO;												// Dummy byte

	cTempAddr[2] = (char) (ftAddress >> 16) | MEM_READ;		// Compose the command and address to send
	cTempAddr[1] = (char) (ftAddress >> 8);								// middle byte
	cTempAddr[0] = (char) (ftAddress);										// low byte

	HAL_GPIO_WritePin(FT800_nCS_GPIO_Port, FT800_nCS_Pin, GPIO_PIN_RESET);	// Set chip select low

  for (i = 2; i >= 0; i--)
	{
		HAL_SPI_Transmit(FT800_hspi, &cTempAddr[i], 1, 100); 			// Send Memory Write plus high address byte
	}

  HAL_SPI_Transmit(FT800_hspi, &cZeroFill, 1, 100);						// Send dummy byte

	for (j = 0; j < sizeof(ftData8); j++)							// Start with least significant byte
	{
		HAL_SPI_TransmitReceive(FT800_hspi,&cZeroFill, &ftData8, 1, 1000);						// Receive data byte	
	}

//	while(SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_BSY) == SET);		//Warten auf letztes Bit
	HAL_GPIO_WritePin(FT800_nCS_GPIO_Port, FT800_nCS_Pin, GPIO_PIN_SET);		// Set chip select high

  return ftData8;																				// Return 8-bits
}

unsigned int ft800memRead16(unsigned long ftAddress)
{
	int i,j;
  unsigned int ftData16;																// 16-bits to return
	unsigned char cTempAddr[3];														// FT800 Memory Address
	unsigned char cTempData[2];														// Place-holder for 16-bits being read
	unsigned char cZeroFill;

	cTempAddr[2] = (char) (ftAddress >> 16) | MEM_READ;		// Compose the command and address to send
	cTempAddr[1] = (char) (ftAddress >> 8);								// middle byte
	cTempAddr[0] = (char) (ftAddress);										// low byte

	HAL_GPIO_WritePin(FT800_nCS_GPIO_Port, FT800_nCS_Pin, GPIO_PIN_RESET);	// Set chip select low

  for (i = 2; i >= 0; i--)
	{
		HAL_SPI_Transmit(FT800_hspi, &cTempAddr[i], 1, 100); 			// Send Memory Write plus high address byte
	}

  HAL_SPI_Transmit(FT800_hspi, &cZeroFill, 1, 100);						// Send dummy byte
	for (j = 0; j < sizeof(cTempData); j++)						// Start with least significant byte
	{
		HAL_SPI_TransmitReceive(FT800_hspi,&cZeroFill, &cTempData[j], 1, 100);				// Receive data byte
	}
//	while(SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_BSY) == SET);		//Warten auf letztes Bit

	HAL_GPIO_WritePin(FT800_nCS_GPIO_Port, FT800_nCS_Pin, GPIO_PIN_SET);		// Set chip select high

	ftData16 = (cTempData[1]<< 8) | 											// Compose value to return - high byte
					   (cTempData[0]); 														// low byte

  return ftData16;																			// Return 16-bits
}

unsigned long ft800memRead32(unsigned long ftAddress)
{
	int i,j;
  unsigned long ftData32;																// 32-bits to return
	unsigned char cTempAddr[3];														// FT800 Memory Address
	unsigned char cTempData[4];														// Place holder for 32-bits being read
	unsigned char cZeroFill;															// Dummy byte

	cTempAddr[2] = (char) (ftAddress >> 16) | MEM_READ;		// Compose the command and address to send
	cTempAddr[1] = (char) (ftAddress >> 8);								// middle byte
	cTempAddr[0] = (char) (ftAddress);										// low byte

	HAL_GPIO_WritePin(FT800_nCS_GPIO_Port, FT800_nCS_Pin, GPIO_PIN_RESET);	// Set chip select low
	
  for (i = 2; i >= 0; i--)
	{
		HAL_SPI_Transmit(FT800_hspi, &cTempAddr[i], 1, 100); 			// Send Memory Write plus high address byte
	}
	
  HAL_SPI_Transmit(FT800_hspi, &cZeroFill, 1, 100);						// Send dummy byte

	for (j = 0; j < sizeof(cTempData); j++)						// Start with least significatn byte
	{
		HAL_SPI_TransmitReceive(FT800_hspi, &cZeroFill, &cTempData[j], 1, 100);				// Receive data byte
	}

//	while(SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_BSY) == SET);		//Warten auf letztes Bit
	
	HAL_GPIO_WritePin(FT800_nCS_GPIO_Port, FT800_nCS_Pin, GPIO_PIN_SET);		// Set chip select high

	ftData32 = (cTempData[3]<< 24) | 											// Compose value to return - high byte
						 (cTempData[2]<< 16) | 
						 (cTempData[1]<< 8) | 
						 (cTempData[0]); 														// Low byte

  return ftData32;																			// Return 32-bits
}

/******************************************************************************
 * Function:        void incCMDOffset(currentOffset, commandSize)
 * PreCondition:    None
 *                    starting a command list
 * Input:           currentOffset = graphics processor command list pointer
 *                  commandSize = number of bytes to increment the offset
 * Output:          newOffset = the new ring buffer pointer after adding the command
 * Side Effects:    None
 * Overview:        Adds commandSize to the currentOffset.  
 *                  Checks for 4K ring-buffer offset roll-over 
 * Note:            None
 *****************************************************************************/
unsigned int incCMDOffset(unsigned int currentOffset, unsigned char commandSize)
{
    unsigned int newOffset;															// Used to hold new offset
    newOffset = currentOffset + commandSize;						// Calculate new offset
    if(newOffset > 4095)																// If new offset past boundary...
    {
        newOffset = (newOffset - 4096);									// ... roll over pointer
    }
	  return newOffset;																		// Return new offset
}

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
