
#include "M95M01.h"
#include "main.h"

/* Private variables ---------------------------------------------------------*/

SPI_HandleTypeDef *EE_hspi;
const char ZeroFill[]={0,0,0,0,0,0,0,0,0,0};
const char *pZeroFill = ZeroFill;

int EE_Init(SPI_HandleTypeDef *hspimain)
{ 
	int status = 0;
	
	EE_hspi = hspimain;
	
/*	
	char bufsave, buf;
	
	EE_Read(0x0000, &bufsave, 1);
	
	const char ctest=0xAA;
	
	EE_Write(0x0000, (char*)&ctest, 1);
	
	EE_Read(0x0000, &buf, 1);
	
	if(buf != ctest)
		status = -1;
	
	EE_Write(0x0000, &bufsave, 1);
*/
    
	return(status);
}

int EE_Read(unsigned long address, char *pBuffer, unsigned int iLength)
{
	unsigned char cmd;
	
	uint32_t ipage; // page counter
	uint32_t page_start_address;

	uint32_t iLengthPage;
	uint8_t *pBufferPage;
	
	for(ipage=page(address); ipage<=page(address+iLength-1); ipage++)
	{	
		if(ipage == page(address))
		{
			page_start_address = address; // for the first page, start at the given address in case we're dropping in to the middle of a page
		} 
		else
		{
			page_start_address = ipage* PAGESIZE; // else we start at the beginning of the current page
		}
		
		//Write in Progress?
		EE_Wait_WIP();
		
		//Set CS low
		EE_CS_ENABLE();
		
		// Send Write Enable Instruction
		cmd=CMD_WREN;
		if(HAL_SPI_Transmit(EE_hspi, &cmd, 1, 0xFFFF) != HAL_OK)
			return 0;

		EE_CS_DISABLE();
		
		EE_CS_ENABLE();
		

		
		// Send Read Instruction
		cmd=CMD_READ;
		if(HAL_SPI_Transmit(EE_hspi, &cmd, 1, 0xFFFF) != HAL_OK)
			return 0;
		//Send Address
		unsigned char cadress[2];
		cadress[0] = (page_start_address >> 8);
		cadress[1] = (page_start_address) ;
		if(HAL_SPI_Transmit(EE_hspi, (uint8_t*)cadress, 2, 0xFFFF) != HAL_OK)
			break;
	
		//Read
		//last page
		if( page(address + iLength) == ipage )
			iLengthPage = address + iLength - page_start_address;
		else // not last page
			iLengthPage = (ipage + 1) * PAGESIZE - page_start_address;
		
		pBufferPage = (uint8_t*) &pBuffer[ page_start_address - address ] ;
		
		if(HAL_SPI_Receive(EE_hspi, pBufferPage, iLengthPage, 0xFFFF) != HAL_OK)
			break;
		
		EE_CS_DISABLE();
	}
	
	EE_CS_DISABLE();
	
	return iLength;	
}

uint32_t page(uint32_t address)
{
	if(address < (uint32_t)PAGESIZE)
	{
		return(0);
	} 
	else 
	{
		return(address / PAGESIZE);
	}
}

uint8_t page_address(uint32_t address)
{

	return(address % PAGESIZE);
}


int EE_Write(unsigned long address, char *pBuffer, unsigned int iLength)
{
	unsigned char cmd;
	
	uint32_t ipage; // page counter
	uint32_t page_start_address;

	uint32_t iLengthPage;
	uint8_t *pBufferPage;
	
	for(ipage=page(address); ipage<=page(address+iLength-1); ipage++)
	{	
		if(ipage == page(address))
		{
			page_start_address = address; // for the first page, start at the given address in case we're dropping in to the middle of a page
		} 
		else
		{
			page_start_address = ipage* PAGESIZE; // else we start at the beginning of the current page
		}
		
		//Write in Progress?
		EE_Wait_WIP();
		
		//Set CS low
		EE_CS_ENABLE();
		
		// Send Write Enable Instruction
		cmd=CMD_WREN;
		if(HAL_SPI_Transmit(EE_hspi, &cmd, 1, 0xFFFF) != HAL_OK)
			return 0;

		EE_CS_DISABLE();
		
		EE_CS_ENABLE();
		
		// Send Write Instruction
		cmd=CMD_WRITE;
		if(HAL_SPI_Transmit(EE_hspi, &cmd, 1, 0xFFFF) != HAL_OK)
			return 0;
		//Send Address
		unsigned char cadress[2];
		cadress[0] = (page_start_address >> 8);
		cadress[1] = (page_start_address) ;
		if(HAL_SPI_Transmit(EE_hspi, (uint8_t*)cadress, 2, 0xFFFF) != HAL_OK)
			break;
		
		//Write Data
		//last page
		if( page(address + iLength) == ipage )
			iLengthPage = address + iLength - page_start_address;
		else // not last page
			iLengthPage = (ipage + 1) * PAGESIZE - page_start_address;
		
		pBufferPage = (uint8_t*) &pBuffer[ page_start_address - address ] ;
		
		if(HAL_SPI_Transmit(EE_hspi, pBufferPage, iLengthPage, 0xFFFF) != HAL_OK)
			break;
		
		EE_CS_DISABLE();
	}
	
	return iLength;	
}

int EE_Wait_WIP(void)
{
	unsigned char cmd;
	unsigned char buf;
	int i=0;
	
	EE_CS_ENABLE();
	
	//Read Status Register
	cmd = CMD_RDSR;
	if(	HAL_SPI_Transmit(EE_hspi, &cmd, 1, 0xFFFF) != HAL_OK)
	{
		EE_CS_DISABLE();
		return -1;
	}
	

	buf = BIT_WIP;
	while( (buf & BIT_WIP) ) //while Write in Progress
	{
		if(HAL_SPI_Receive(EE_hspi, (uint8_t*) &buf, 1, 0xFFFF) != HAL_OK)
		{
			EE_CS_DISABLE();
			return -1;
		}
		i++;
	}
	
	EE_CS_DISABLE();
	
	return i;
}

int	EE_GetRegisterValue(unsigned char cmd, unsigned char * pReadData, int iNum)
{
	HAL_StatusTypeDef Result;
	
	//Set CS low
	EE_CS_ENABLE();
	
	HAL_SPI_Transmit(EE_hspi, &cmd, 1, 100); 			// Send Command
	
	Result = HAL_SPI_TransmitReceive(EE_hspi, (uint8_t*) pZeroFill, pReadData, iNum, 100);
	
	EE_CS_DISABLE();
	
	if(Result == HAL_OK) 			// Read Data
		return iNum;
	else
		return 0;
}


