/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2020 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "fatfs.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include <string.h>
#include "AD7794.h"
#include "FT800.h"
#include "DS18B20.h"
#include "M95M01.h"
#include "sim800.h"
#include "menu.h"
#include "AM2302.h"
#include <math.h>

/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
#define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
ADC_HandleTypeDef hadc1;

SPI_HandleTypeDef hspi1;
SPI_HandleTypeDef hspi2;

UART_HandleTypeDef huart1;
UART_HandleTypeDef huart2;

char AD7794NewConversion;
char bAD7794ConvReady, bDS18ConvReady, bDHTConvReady, bVBatADCConvReady; 

/* USER CODE BEGIN PV */

/* Private variables ---------------------------------------------------------*/
//USERFatFS and USERFile declared in fatfs.c and as extern in fatfs.h

enumModetype Mode = ModeActive;

char bEXTI=0;

//Tasks
static uint32_t Tick100ms, Tick1s, Tick60s;
static uint32_t thingspeakmin=0;

//FAT
FATFS *pfs;
FRESULT fres;
DWORD fre_clust;
uint32_t total, free;
char buffer[100];

long datasum; //Raw ADC Data
float fweight1=0.0f;
float fweightcalibrated=0.0f;
float ftemp1=0.0f, ftemp2=0.0f, ftemp3=0.0f;
float DHTTemp, DHTHumidity;

int iBatVoltageRaw;
float fBatVoltage;

//PWMOut;
structPWMOut PWMOut;

//PID
structPID PID;

//Log
structLogtoFile Log;

//Display
char pressedtag;
extern char newtag;			// Pressed Touch Tag
extern char Refresh;		// Trigger to Refresh Display Content

char bSDDebug = FALSE;

char CommandBuffer[100];
char *pCommandBuffer;
char bCommandBuffer;
char bCommandEnter;
char bCommandOverFlow;
char bSIM800Debug=FALSE;


char SIM800Buffer[100];
char *pSIM800Buffer;
char *pSIM800LastCommand;
char bSIM800Buffer;
char bSIM800Enter;
char bSIM800OverFlow;

RTCTIME rtcTime;

structData Datatemp = {DATASIZE};
structData Dataw = {DATASIZE};
structData Dataweight = {DATASIZE};

structSettings Settings;

structSettingsIndex SettingsIndex[] ={
//SMS
{ Settingssz, "SMS Telephone Number", &Settings.SMSNumber},
//Weight
{ Settingschar, "ADC Speed Setting", &Settings.ADCSpeedSetting},
{ Settingslong, "ADC d Offset", &Settings.lOffset},
{ Settingsdouble, "ADC k Factor", &Settings.dFactor},
{ Settingsfloat, "Weight Kalibriergewicht", &Settings.fCalWeight},
{ Settingsfloat, "Weight Referenztemperatur", &Settings.fCalTemp},
{ Settingsfloat, "Weight dw/dt (kg/K)", &Settings.fdwdt},
{ Settingsfloat, "Weight Offset", &Settings.fWeightOffset},

//Touch
{ Settingsulong, "Touch Transform Register A", &Settings.ulSET_REG_CTOUCH_TRANSFORM_A},
{ Settingsulong, "Touch Transform Register B", &Settings.ulSET_REG_CTOUCH_TRANSFORM_B},
{ Settingsulong, "Touch Transform Register C", &Settings.ulSET_REG_CTOUCH_TRANSFORM_C},
{ Settingsulong, "Touch Transform Register D", &Settings.ulSET_REG_CTOUCH_TRANSFORM_D},
{ Settingsulong, "Touch Transform Register E", &Settings.ulSET_REG_CTOUCH_TRANSFORM_E},
{ Settingsulong, "Touch Transform Register F", &Settings.ulSET_REG_CTOUCH_TRANSFORM_F},

//PID
//{ Settingsfloat, "PID kr", &Settings.PID.kr},
//{ Settingsfloat, "PID tN", &Settings.PID.tN},
//{ Settingsfloat, "PID tV", &Settings.PID.tV},
//{ Settingsfloat, "PID t1", &Settings.PID.t1},
//{ Settingsfloat, "PID T Periode", &Settings.PID.T},
//{ Settingsfloat, "PID tPWM", &Settings.PID.tPWM},
//{ Settingsfloat, "Soll Temperature", &Settings.fTempSet},
//{ Settingsfloat, "Override PID Output Value", &Settings.fueOverride},

//ThingSpeak
{ Settingschar, "thingspeak Log", &Settings.LogToThingSpeak},
{ Settingsint,  "thingspeak Interval [min]", &Settings.LogToThingSpeakInterval},
{ Settingssz,   "thingspeak key", &Settings.ThingSpeakKey},
//SD
{ Settingschar, "SD Card Log", &Settings.LogToSD},
//Email
{ Settingssz, "SMTP Server", &Settings.SMTPServer},
{ Settingssz, "SMTPUser", &Settings.SMTPUser},
{ Settingssz, "SMTP Password", &Settings.SMTPPass},
{ Settingssz, "Email From", &Settings.EmailFrom},
{ Settingssz, "From Name", &Settings.FromName},
{ Settingssz, "Email To", &Settings.EmailTo},
{ Settingssz, "To Name", &Settings.ToName},

//Diagram
{ Settingsfloat, "Diagram Min", &Settings.DiagramMinY},
{ Settingsfloat, "Diagram Max", &Settings.DiagramMaxY},

//SMSAlarm
{ Settingschar, "SMS Alarm OFF=0", &Settings.cSMSAlarm},
{ Settingsfloat, "Deviation for SMS Alarm", &Settings.fAlarmDiff},

{ Settingsfloat, "Weight bevor work", &Settings.fWeightBevor},

};

int GetSizeofSettingsIndex(void)
{
	return (sizeof(SettingsIndex)/sizeof(SettingsIndex[0]));
}

structPWRStatus PWRStatus={0,0,0,0};

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_ADC1_Init(void);
static void MX_RTC_Init(void);
static void MX_SPI1_Init(void);
static void MX_SPI2_Init(void);
static void MX_USART1_UART_Init(void);
static void MX_USART2_UART_Init(void);
/* USER CODE BEGIN PFP */
int StopMode(void);
int PowerOn(void);

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */
	
	
  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_ADC1_Init();
  MX_RTC_Init();
  MX_SPI1_Init();
  MX_SPI2_Init();
  MX_USART1_UART_Init();
  MX_USART2_UART_Init();
  MX_FATFS_Init();
  /* USER CODE BEGIN 2 */

	//All CS high
	HAL_GPIO_WritePin(SD_nCS_GPIO_Port, SD_nCS_Pin, GPIO_PIN_SET);
	HAL_GPIO_WritePin(FT800_nCS_GPIO_Port, FT800_nCS_Pin, GPIO_PIN_SET);
	
	LED_Green_ON();
	LED_Red_OFF();

	//Debug UART1
	printf("\nBoot...\n");
	
	pCommandBuffer = CommandBuffer;
	bCommandBuffer = 0;
	bCommandOverFlow = 0;

	HAL_UART_Receive_IT(&huart1, (uint8_t*) pCommandBuffer, 1); // Start Receiving on uart1
	
	pSIM800Buffer = SIM800Buffer;
	pSIM800LastCommand = SIM800Buffer;
	bSIM800Buffer = 0;
	bSIM800OverFlow = 0;
	HAL_UART_Receive_IT(&huart2, (uint8_t*) pSIM800Buffer, 1); //Start Receiving on uart2

	PWR_3_3VD_ON();
	
	PWR_FT800_ON();
	PWR_BACKLIGHT_ON();
	PWR_4V_ON();
	PWRStatus.PWRSIM800 = 1;
	
	PWR_SD_ON();
	PWRStatus.PWRSD=1;

	char error;
	int result;

	//EEPROM
	printf("Init EEPROM M95\n");
	result = EE_Init(&hspi2);
	if(!result)
		printf("M95M01 O.K.\n");
	else
		printf("Fault M95M01\n");
	
	InitSettings(&Settings);
	PrintSettings(&Settings);

	//FT800
	Menu_Init();
	//SPI Enable
	FT800_SPI_ENABLE();

	printf("Init FT800\n");
	error = FT800_Init(&hspi2);
	if(error)
	{
		printf("ID FT800 O.K.\n");
		PWRStatus.PWRFT800 = 1;
	}
	else
		printf("Fault ID FT800\n");

	FT800_SPI_DISABLE();
	
	InitDiagram(&Diagram, &Dataweight, &Datatemp);
	
	InitData(&Datatemp);
	InitData(&Dataweight);
	
	//AD7794
	printf("Init AD7794\n");
	AD7794NewConversion=1;
	PWR_AVBAT_ON();
	PWRStatus.PWRAnalog=1;
	HAL_Delay(2);
	ADI_PART_CS_HIGH;
	HAL_Delay(10);
	error = AD7794_Init(&hspi1);
	if(error)
		printf("ID AD7794 O.K.\n");
	else
		printf("Fault ID AD7794\n");
	AD7794_SetGain(5);
	AD7794_SetIntReference(0);
	AD7794_SetChannel(0);
	AD7794_SetMode(AD7794_MODE_CONT );
	AD7794_SetRate(Settings.ADCSpeedSetting);
	ADI_PART_CS_LOW; //To sense RDY Pin
	
	//DS18B20
	DS18Init(DS_1W_1_GPIO_Port, DS_1W_1_Pin);
	printf("Init DS18B20 CH1\n");
	if(ow_reset()==0)
	{
		printf("DS18B20 CH1 present\n");
		Read_ROMCode();
	}
	else
		printf("DS18B20 CH1 not present\n");

	DS18Init(DS_1W_1_GPIO_Port, DS_1W_2_Pin);
	printf("Init DS18B20 CH2\n");
	if(ow_reset()==0)
	{
		printf("DS18B20 CH2 present\n");
		Read_ROMCode();
	}
	else
		printf("DS18B20 CH2 not present\n");

		DS18Init(DS_1W_1_GPIO_Port, DS_1W_3_Pin);
	printf("Init DS18B20 CH3\n");
	if(ow_reset()==0)
	{
		printf("DS18B20 CH3 present\n");
		Read_ROMCode();
	}
	else
		printf("DS18B20 CH3 not present\n");

	
	//AM2302
	printf("Init AM2302\n");
	DHTInit(GPIOC, GPIO_PIN_9, AM2302, 30);
	DHTHumidity = DHTreadHumidity();
	DHTTemp = DHTreadTemperature(FALSE);
	printf("Read DHT Temp %f C\n",DHTHumidity);
	printf("Read DHT Humidity %f %%\n",DHTTemp);
	
	//SDCARD
	printf("Init SDCARD\n");
	Init_SD();

	//RTC Init
	printf("Init RTC\n");
	
	rtc_gettime(&rtcTime);
	printf("Time %02u:%02u:%02u\n",rtcTime.hour, rtcTime.min, rtcTime.sec);
	printf("Date %02u.%02u.%04u\n", rtcTime.mday, rtcTime.month, rtcTime.year);
	
	//ADC
	printf("InitADC\n");
	if(HAL_ADC_Start(&hadc1)!=HAL_OK)
		printf("ADC Error\n");

	if(HAL_ADCEx_Calibration_Start(&hadc1)!=HAL_OK)
		printf("ADC_Calibration_Error");

	if(HAL_ADC_Start(&hadc1)!=HAL_OK)
		printf("ADC Error\n");

	HAL_ADC_PollForConversion(&hadc1,1000);
	
	iBatVoltageRaw = HAL_ADC_GetValue(&hadc1);
	fBatVoltage = iBatVoltageRaw * 0.00912f;
	printf("VBat=%6.3f\n",fBatVoltage);
	if(HAL_ADC_Start(&hadc1)!=HAL_OK)
		printf("ADC Error\n");
	
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
	Tick60s = Tick1s = Tick100ms = HAL_GetTick();
/*
	InitPID(&PID);
	PIDSetw(&PID, Settings.fTempSet);
	ftemp1 = Settings.fTempSet;

	PID.ulTick = HAL_GetTick();
	
	InitPWMOut(&PWMOut);
	PWMOut.ulTick = HAL_GetTick();
*/	
	if(Settings.LogToSD)
		if(InitLogtoFile(&Log))
			printf("InitLogtoFile Error\n");

	if(Settings.LogToThingSpeak)
	{
		MessageBoxText("Start SIM800", 1);
		sim800on();
		MessageBoxText(0,0);
	}

		
  while (1)
  {
	
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
		
		if(bEXTI)
		{
			bEXTI=0;
			printf("EXTI\n");
			if(Mode == ModeActive)
			{
				Mode = ModeLog;
				printf("ModeLog\n");
				StopMode();
			}
			else
			{
				Mode = ModeActive;
				printf("ModeActive\n");
				PowerOn();
			}
		}
		//Active Mode
		if(Mode==ModeActive)
		{

			CriticalTasks();
			
			NonCriticalTasks();
			
			DebugMonitorTasks();
		} else
		//LogMode
		{
			AD7794NewConversion=1;
			
			CriticalTasks();
			
			DebugMonitorTasks();
			
			//Wait for Conversion
			if(bAD7794ConvReady == bDS18ConvReady == bDHTConvReady == bVBatADCConvReady == 1)
			{
				//Diagram
				AddData(&Dataweight,fweightcalibrated);

				//LogToThingspeak
				printf("Log Weight %f Temp1 %f Temp2 %f DHTTemp %f DHTHumidity %f VBat %f\n", fweightcalibrated, ftemp1, ftemp2, DHTTemp, DHTHumidity, fBatVoltage);

				//SMS Alarm
// 			if(Settings.cSMSAlarm)
// 			{
// 				if ( ((ftemp1 - PID.w) > Settings.fAlarmDiff) || ((ftemp1 - PID.w) < (-Settings.fAlarmDiff)) )
// 				{	//SendSMS
// 					MessageBoxText("Send Altert SMS", 1);
// 					snprintf(sztemp,19, "Alert Temp=%5.2f", ftemp1);
// 					SendSMS(sztemp);
// 					MessageBoxText(0, 0);					
// 				}
// 			}

				//Thingspeak
				MessageBoxText("Send thingspeak", 1);
				thingspeakmin=0;
				if(Settings.LogToThingSpeak)
				{
					for(char i=0; i<10; i++) // 10 Versuche
					{
						HAL_Delay(1000);
						if(sendthingspeak()==0)
							break; //success
					}
				}
				MessageBoxText(0, 0);
				
				//Sleep
				StopMode();
			}
		}
		
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
  RCC_PeriphCLKInitTypeDef PeriphClkInit = {0};

  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE|RCC_OSCILLATORTYPE_LSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.HSEPredivValue = RCC_HSE_PREDIV_DIV1;
  RCC_OscInitStruct.LSEState = RCC_LSE_ON;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLMUL = RCC_PLL_MUL9;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
  {
    Error_Handler();
  }
  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_RTC|RCC_PERIPHCLK_ADC;
  PeriphClkInit.RTCClockSelection = RCC_RTCCLKSOURCE_LSE;
  PeriphClkInit.AdcClockSelection = RCC_ADCPCLK2_DIV6;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief ADC1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_ADC1_Init(void)
{

  /* USER CODE BEGIN ADC1_Init 0 */

  /* USER CODE END ADC1_Init 0 */

  ADC_ChannelConfTypeDef sConfig = {0};

  /* USER CODE BEGIN ADC1_Init 1 */

  /* USER CODE END ADC1_Init 1 */
  /** Common config 
  */
  hadc1.Instance = ADC1;
  hadc1.Init.ScanConvMode = ADC_SCAN_DISABLE;
  hadc1.Init.ContinuousConvMode = DISABLE;
  hadc1.Init.DiscontinuousConvMode = DISABLE;
  hadc1.Init.ExternalTrigConv = ADC_SOFTWARE_START;
  hadc1.Init.DataAlign = ADC_DATAALIGN_RIGHT;
  hadc1.Init.NbrOfConversion = 1;
  if (HAL_ADC_Init(&hadc1) != HAL_OK)
  {
    Error_Handler();
  }
  /** Configure Regular Channel 
  */
  sConfig.Channel = ADC_CHANNEL_14;
  sConfig.Rank = ADC_REGULAR_RANK_1;
  sConfig.SamplingTime = ADC_SAMPLETIME_1CYCLE_5;
  if (HAL_ADC_ConfigChannel(&hadc1, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN ADC1_Init 2 */

  /* USER CODE END ADC1_Init 2 */

}

/**
  * @brief RTC Initialization Function
  * @param None
  * @retval None
  */
static void MX_RTC_Init(void)
{

  /* USER CODE BEGIN RTC_Init 0 */

  /* USER CODE END RTC_Init 0 */

  LL_RTC_InitTypeDef RTC_InitStruct = {0};

    LL_PWR_EnableBkUpAccess();
    /* Enable BKP CLK enable for backup registers */
    LL_APB1_GRP1_EnableClock(LL_APB1_GRP1_PERIPH_BKP);
  /* Peripheral clock enable */
  LL_RCC_EnableRTC();

  /* RTC interrupt Init */
  NVIC_SetPriority(RTC_IRQn, NVIC_EncodePriority(NVIC_GetPriorityGrouping(),0, 0));
  NVIC_EnableIRQ(RTC_IRQn);

  /* USER CODE BEGIN RTC_Init 1 */
	  /* EXTI interrupt init*/
	SET_BIT(EXTI->IMR, EXTI_IMR_MR17);
	SET_BIT(EXTI->RTSR, EXTI_IMR_MR17);
  NVIC_SetPriority(RTC_Alarm_IRQn, 0);
  NVIC_EnableIRQ(RTC_Alarm_IRQn);

  /* USER CODE END RTC_Init 1 */
  /** Initialize RTC and set the Time and Date 
  */
  RTC_InitStruct.AsynchPrescaler = 32768;
  LL_RTC_Init(RTC, &RTC_InitStruct);
  LL_RTC_SetAsynchPrescaler(RTC, 32768);
  /* USER CODE BEGIN RTC_Init 2 */

  /* USER CODE END RTC_Init 2 */

}

/**
  * @brief SPI1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_SPI1_Init(void)
{

  /* USER CODE BEGIN SPI1_Init 0 */

  /* USER CODE END SPI1_Init 0 */

  /* USER CODE BEGIN SPI1_Init 1 */

  /* USER CODE END SPI1_Init 1 */
  /* SPI1 parameter configuration*/
  hspi1.Instance = SPI1;
  hspi1.Init.Mode = SPI_MODE_MASTER;
  hspi1.Init.Direction = SPI_DIRECTION_2LINES;
  hspi1.Init.DataSize = SPI_DATASIZE_8BIT;
  hspi1.Init.CLKPolarity = SPI_POLARITY_HIGH;
  hspi1.Init.CLKPhase = SPI_PHASE_2EDGE;
  hspi1.Init.NSS = SPI_NSS_SOFT;
  hspi1.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_16;
  hspi1.Init.FirstBit = SPI_FIRSTBIT_MSB;
  hspi1.Init.TIMode = SPI_TIMODE_DISABLE;
  hspi1.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  hspi1.Init.CRCPolynomial = 10;
  if (HAL_SPI_Init(&hspi1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN SPI1_Init 2 */

  /* USER CODE END SPI1_Init 2 */

}

/**
  * @brief SPI2 Initialization Function
  * @param None
  * @retval None
  */
static void MX_SPI2_Init(void)
{

  /* USER CODE BEGIN SPI2_Init 0 */

  /* USER CODE END SPI2_Init 0 */

  /* USER CODE BEGIN SPI2_Init 1 */

  /* USER CODE END SPI2_Init 1 */
  /* SPI2 parameter configuration*/
  hspi2.Instance = SPI2;
  hspi2.Init.Mode = SPI_MODE_MASTER;
  hspi2.Init.Direction = SPI_DIRECTION_2LINES;
  hspi2.Init.DataSize = SPI_DATASIZE_8BIT;
  hspi2.Init.CLKPolarity = SPI_POLARITY_LOW;
  hspi2.Init.CLKPhase = SPI_PHASE_1EDGE;
  hspi2.Init.NSS = SPI_NSS_SOFT;
  hspi2.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_4;
  hspi2.Init.FirstBit = SPI_FIRSTBIT_MSB;
  hspi2.Init.TIMode = SPI_TIMODE_DISABLE;
  hspi2.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  hspi2.Init.CRCPolynomial = 10;
  if (HAL_SPI_Init(&hspi2) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN SPI2_Init 2 */

  /* USER CODE END SPI2_Init 2 */

}

/**
  * @brief USART1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_USART1_UART_Init(void)
{

  /* USER CODE BEGIN USART1_Init 0 */

  /* USER CODE END USART1_Init 0 */

  /* USER CODE BEGIN USART1_Init 1 */

  /* USER CODE END USART1_Init 1 */
  huart1.Instance = USART1;
  huart1.Init.BaudRate = 115200;
  huart1.Init.WordLength = UART_WORDLENGTH_8B;
  huart1.Init.StopBits = UART_STOPBITS_1;
  huart1.Init.Parity = UART_PARITY_NONE;
  huart1.Init.Mode = UART_MODE_TX_RX;
  huart1.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart1.Init.OverSampling = UART_OVERSAMPLING_16;
  if (HAL_UART_Init(&huart1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN USART1_Init 2 */

  /* USER CODE END USART1_Init 2 */

}

/**
  * @brief USART2 Initialization Function
  * @param None
  * @retval None
  */
static void MX_USART2_UART_Init(void)
{

  /* USER CODE BEGIN USART2_Init 0 */

  /* USER CODE END USART2_Init 0 */

  /* USER CODE BEGIN USART2_Init 1 */

  /* USER CODE END USART2_Init 1 */
  huart2.Instance = USART2;
  huart2.Init.BaudRate = 115200;
  huart2.Init.WordLength = UART_WORDLENGTH_8B;
  huart2.Init.StopBits = UART_STOPBITS_1;
  huart2.Init.Parity = UART_PARITY_NONE;
  huart2.Init.Mode = UART_MODE_TX_RX;
  huart2.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart2.Init.OverSampling = UART_OVERSAMPLING_16;
  if (HAL_UART_Init(&huart2) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN USART2_Init 2 */

  /* USER CODE END USART2_Init 2 */

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOD_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOC, FT800_SPI_nEN_Pin|SIM800_PWRKEY_Pin|FT800_nEN_PWR_Pin, GPIO_PIN_SET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOC, EN_4V_Pin|DS_1W_1_Pin|DS_1W_2_Pin|DS_1W_3_Pin 
                          |DS_1W_4_Pin|FT800_nPD_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(SIM800_DTR_GPIO_Port, SIM800_DTR_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOA, AD7794_nCS_Pin|SD_nEN_PWR_Pin|FT800_nCS_Pin|SD_SPI_nEN_Pin, GPIO_PIN_SET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOB, EN_AVBAT_Pin|OUT1_Pin|OUT2_Pin|EN_3_3VD_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOB, LED_GREEN_Pin|LED_RED_Pin|EEPROM_nCS_Pin|SD_nCS_Pin, GPIO_PIN_SET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(BACKLIGHT_nEN_PWR_GPIO_Port, BACKLIGHT_nEN_PWR_Pin, GPIO_PIN_SET);

  /*Configure GPIO pin : FT800_SPI_nEN_Pin */
  GPIO_InitStruct.Pin = FT800_SPI_nEN_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(FT800_SPI_nEN_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pins : EN_4V_Pin FT800_nEN_PWR_Pin FT800_nPD_Pin */
  GPIO_InitStruct.Pin = EN_4V_Pin|FT800_nEN_PWR_Pin|FT800_nPD_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

  /*Configure GPIO pins : SIM800_PWRKEY_Pin DS_1W_1_Pin DS_1W_2_Pin DS_1W_3_Pin 
                           DS_1W_4_Pin */
  GPIO_InitStruct.Pin = SIM800_PWRKEY_Pin|DS_1W_1_Pin|DS_1W_2_Pin|DS_1W_3_Pin 
                          |DS_1W_4_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_OD;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

  /*Configure GPIO pin : SIM800_RING_Pin */
  GPIO_InitStruct.Pin = SIM800_RING_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(SIM800_RING_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pin : SYS_WKUP_Pin */
  GPIO_InitStruct.Pin = SYS_WKUP_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(SYS_WKUP_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pins : SIM800_DTR_Pin AD7794_nCS_Pin SD_nEN_PWR_Pin FT800_nCS_Pin 
                           SD_SPI_nEN_Pin */
  GPIO_InitStruct.Pin = SIM800_DTR_Pin|AD7794_nCS_Pin|SD_nEN_PWR_Pin|FT800_nCS_Pin 
                          |SD_SPI_nEN_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pins : PB0_Pin PB1_Pin SD_nUNLOCK_Pin */
  GPIO_InitStruct.Pin = PB0_Pin|PB1_Pin|SD_nUNLOCK_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /*Configure GPIO pins : EN_AVBAT_Pin LED_GREEN_Pin LED_RED_Pin EEPROM_nCS_Pin 
                           OUT1_Pin OUT2_Pin EN_3_3VD_Pin SD_nCS_Pin */
  GPIO_InitStruct.Pin = EN_AVBAT_Pin|LED_GREEN_Pin|LED_RED_Pin|EEPROM_nCS_Pin 
                          |OUT1_Pin|OUT2_Pin|EN_3_3VD_Pin|SD_nCS_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /*Configure GPIO pin : PC11 */
  GPIO_InitStruct.Pin = GPIO_PIN_11;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

  /*Configure GPIO pin : BACKLIGHT_nEN_PWR_Pin */
  GPIO_InitStruct.Pin = BACKLIGHT_nEN_PWR_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(BACKLIGHT_nEN_PWR_GPIO_Port, &GPIO_InitStruct);

  /* EXTI interrupt init*/
  HAL_NVIC_SetPriority(EXTI0_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(EXTI0_IRQn);

  HAL_NVIC_SetPriority(EXTI15_10_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(EXTI15_10_IRQn);

}

/* USER CODE BEGIN 4 */

int CriticalTasks(void)
{

// 	//PID
// 	if(HAL_GetTick() - PID.ulTick > PID.ulPeriode)
// 	{
// 		PID.ulTick = HAL_GetTick();
// 		
// 		//Calculate New Value
// 		if((ftemp1>(-20.0f)) && (ftemp1<50.0f))
// 		{
// 			PIDSeten(&PID, ftemp1);
// 			CalculatePID(&PID);
// 			SetPWMOutValue(&PWMOut, PID.un);
// 		}
// 		
// 		//Log Temp
// 		if(Settings.LogToSD)
// 			LogtoFile(&Log);
// 		
// 		//Data
// 		AddData(&Datatemp,ftemp1);
// 		AddData(&Dataw, DHTHumidity);
// 	}
// 	
// 	//PWM Out
// 	if(HAL_GetTick() - PWMOut.ulTick > PWMOut.ulToggle)
// 	{
// 		//Output off
// 			HAL_GPIO_WritePin(OUT1_GPIO_Port, OUT1_Pin, GPIO_PIN_RESET);
// 	}
// 	if(HAL_GetTick() - PWMOut.ulTick > PWMOut.ulPeriode)
// 	{
// 		PWMOut.ulTick = HAL_GetTick();
// 		
// 		if(PWMOut.ulToggle > 0)
// 			//Output on
// 			HAL_GPIO_WritePin(OUT1_GPIO_Port, OUT1_Pin, GPIO_PIN_SET);
// 		else
// 			//Output off
// 			HAL_GPIO_WritePin(OUT1_GPIO_Port, OUT1_Pin, GPIO_PIN_RESET);
// 	}
	

	//100ms Thread
	if ((HAL_GetTick() - Tick100ms) > 50)
	{
		LED_Green_ON();
		Tick100ms = HAL_GetTick();

		//Display
		if(PWRStatus.PWRFT800==1)
		{
			FT800_SPI_ENABLE();
			pressedtag = FT800_Read_Tag();
			FT800_SPI_DISABLE();

			if (pressedtag>0 && pressedtag < 7)
			{
				Refresh=1;
				Menu_Press(pressedtag);
			}
			if(newtag==20)
			{
				FT800_SPI_ENABLE();
				Diagram_Press();
				Refresh=1;
				FT800_SPI_DISABLE();
			}
		}

		LED_Green_OFF();
	}
	
	if(PWRStatus.PWRFT800==1 && Refresh)
	{
		LED_Green_ON();

		FT800_SPI_ENABLE();
		Draw_Display(pressedtag);
		Refresh = 0;
		FT800_SPI_DISABLE();
		LED_Green_OFF();

	}	

	//AD7794 Conversion Ready?
	if( AD7794_RDY_STATE == 0)
	{
		static long data[8];
		static unsigned char i;
		char ii;
//		static float fweightold;

		LED_Green_ON();
		
		if(AD7794NewConversion)
		{
			AD7794NewConversion=0;
			i=0;
		}
		else
		{
			i++;
			if(i>7)
			{
				bAD7794ConvReady = 1;
				i=0;
			}
		}	
		
		//Read Data
		data[i] = AD7794_ReadConversion();
		
		datasum = 0;
		
		for(ii=0; ii<8; ii++)
			datasum+=data[ii];
		
		datasum /=8;
		
		fweight1 = (datasum - Settings.lOffset) * Settings.dFactor;		//Boxcar Filter

//		fweight1 = (data[i] - Settings.lOffset) * Settings.dFactor;	//No Filter
		
		ADI_PART_CS_LOW;
		if (bAD7794ConvReady)
			fweightcalibrated = fweight1 + (ftemp1-Settings.fCalTemp)*Settings.fdwdt - Settings.fWeightOffset;
		else
			fweightcalibrated = 0.0f;
		
//		AddData(&Dataweight,fweight1);
//		if(fabs(fweightold - fweight1) >= 0.001)
//		{
			Refresh=1;
//			fweightold = fweight1;
//		}

		LED_Green_OFF();
	}
	
		
	//1 sec Thread
	if ((HAL_GetTick() - Tick1s) > 1000)
	{	
		LED_Green_ON();
		Tick1s = HAL_GetTick();
		if(PWRStatus.PWRAnalog==1)
		{
			bDS18ConvReady = 1;
			
			DS18Init(DS_1W_1_GPIO_Port, DS_1W_1_Pin);				
			ftemp1 = Read_Temperature();
			Start_Conversion();

			DS18Init(DS_1W_1_GPIO_Port, DS_1W_2_Pin);				
			ftemp2 = Read_Temperature();
			Start_Conversion();
		
			//DHT
			bDHTConvReady=1;
			DHTHumidity = DHTreadHumidity();
			DHTTemp = DHTreadTemperature(FALSE);

			//VBat ADC
			HAL_ADC_PollForConversion(&hadc1,1000);
			iBatVoltageRaw = HAL_ADC_GetValue(&hadc1);
			fBatVoltage = iBatVoltageRaw * 0.00912f;
			bVBatADCConvReady=1;
			if(HAL_ADC_Start(&hadc1)!=HAL_OK)
				printf("ADC Error\n");
		}	
		
		rtc_gettime(&rtcTime);
		
		Refresh = 1;

		LED_Green_OFF();
	}
	
				
		return 0;
}

int NonCriticalTasks(void)
{
//	char sztemp[20];

	//60 sec Thread
	if ((HAL_GetTick() - Tick60s) > 60000UL)
	{	
		LED_Green_ON();
		
		Tick60s = HAL_GetTick();
		thingspeakmin++;
		if(thingspeakmin >= Settings.LogToThingSpeakInterval)
		{
			//SMS Alarm
// 			if(Settings.cSMSAlarm)
// 			{
// 				if ( ((ftemp1 - PID.w) > Settings.fAlarmDiff) || ((ftemp1 - PID.w) < (-Settings.fAlarmDiff)) )
// 				{	//SendSMS
// 					MessageBoxText("Send Altert SMS", 1);
// 					snprintf(sztemp,19, "Alert Temp=%5.2f", ftemp1);
// 					SendSMS(sztemp);
// 					MessageBoxText(0, 0);					
// 				}
// 			}
			
			//Diagram
			AddData(&Dataweight,fweightcalibrated);

			//Thingspeak
			MessageBoxText("Send thingspeak", 1);

			thingspeakmin=0;
			if(Settings.LogToThingSpeak)
				sendthingspeak();
			
			MessageBoxText(0, 0);
		}
		LED_Green_OFF();
	}

	return 0;
}

int DebugMonitorTasks(void)
{
		char error;

	//Received Byte from SIM800
	if(bSIM800Buffer)
	{
		LED_Green_ON();
		bSIM800Buffer = 0;

		HAL_UART_Transmit(&huart1, (uint8_t*) pSIM800LastCommand, 1, 1000);
		
		if( pSIM800LastCommand++ >= (SIM800Buffer + sizeof(SIM800Buffer) - 1 ))
			pSIM800LastCommand = SIM800Buffer;

		if(pSIM800LastCommand != pSIM800Buffer)
			bSIM800Buffer = 1;
		
		LED_Green_OFF();
	}
	
	if(bCommandBuffer)
	{
		LED_Green_ON();
		bCommandBuffer = 0;
		if(bCommandEnter)		//Received '\r'
		{
			char bCommandDone=0;
			
			bCommandEnter = 0;

			printf("\n");
			if(strncmp(CommandBuffer, "touchcal",8) == 0)
			{
				printf("Calibrate touch\n");
				FT800_SPI_ENABLE();
				FT800_Calibrate();
				unsigned long temp;
				temp=ft800memRead32(REG_CTOUCH_TRANSFORM_A);
				Settings.ulSET_REG_CTOUCH_TRANSFORM_A = temp;
				printf("TRANSFORM_A %u\n",temp);
				temp=ft800memRead32(REG_CTOUCH_TRANSFORM_B);
				Settings.ulSET_REG_CTOUCH_TRANSFORM_B = temp;
				printf("TRANSFORM_B %u\n",temp);
				temp=ft800memRead32(REG_CTOUCH_TRANSFORM_C);
				Settings.ulSET_REG_CTOUCH_TRANSFORM_C = temp;
				printf("TRANSFORM_C %u\n",temp);
				temp=ft800memRead32(REG_CTOUCH_TRANSFORM_D);
				Settings.ulSET_REG_CTOUCH_TRANSFORM_D = temp;
				printf("TRANSFORM_D %u\n",temp);
				temp=ft800memRead32(REG_CTOUCH_TRANSFORM_E);
				Settings.ulSET_REG_CTOUCH_TRANSFORM_E = temp;
				printf("TRANSFORM_E %u\n",temp);
				temp=ft800memRead32(REG_CTOUCH_TRANSFORM_F);
				Settings.ulSET_REG_CTOUCH_TRANSFORM_F = temp;
				printf("TRANSFORM_F %u\n",temp);
				FT800_SPI_DISABLE();
				SaveSettings(&Settings);
				printf("End Calibration\n");
			}

			if(strncmp(CommandBuffer, "sddebug",7) == 0)
			{
				bSDDebug=1;
				
				SD_SPI_ENABLE();
					
				//Clock Polarity
				hspi2.Init.CLKPolarity = SPI_POLARITY_HIGH;
				hspi2.Init.CLKPhase = SPI_PHASE_2EDGE;
				hspi2.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_128;
				if (HAL_SPI_Init(&hspi2) != HAL_OK)
				{
					Error_Handler();
				}
				SDDebug();
			}

			if(strncmp(CommandBuffer, "alloff",7) == 0)
			{
				//SIM800 off
				printf("SIM800 Port off\n");
				SIM800_Port_OFF();
				printf("SIM800 Power off\n");
				PWR_4V_OFF();	
				PWRStatus.PWRSIM800=0;
				//FT800 off
				printf("Backlight off\n");
				PWR_BACKLIGHT_OFF();
				printf("FT800 Port off\n");
				FT800_Port_OFF();
				printf("PWR_FT800_OFF\n");
				PWR_FT800_OFF();
				PWRStatus.PWRFT800=0;
				//SD off
				printf("SD Port off\n");
				SD_Port_OFF();
				printf("SD Power off\n");
				PWR_SD_OFF();

				printf("3.3VD OFF\n");
				PWR_3_3VD_OFF();
				//Analog OFF
				printf("Analog OFF\n");
				Analog_Port_OFF();
				PWRStatus.PWRAnalog=0;
				PWR_AVBAT_OFF();
				
				printf("DeInit ADC\n");
				HAL_ADC_DeInit(&hadc1);
			}
			
			if(strncmp(CommandBuffer, "allon",7) == 0)
			{
				printf("3.3VD ON\n");
				PWR_3_3VD_ON();
				
				printf("Init ADC\n");
				HAL_ADC_Init(&hadc1);
				
				//SIM800 on
				printf("SIM800 Power on\n");
				PWR_4V_ON();					
				printf("SIM800 Port on\n");
				SIM800_Port_ON();
				PWRStatus.PWRSIM800=1;
				//FT800 on
				printf("PWR_FT800_ON\n");
				PWR_FT800_ON();
				printf("FT800 Port on\n");
				FT800_Port_ON();
				printf("Backlight on\n");
				PWR_BACKLIGHT_ON();
				FT800_SPI_ENABLE();
				printf("Init FT800\n");
				error = FT800_Init(&hspi2);
				if(error)
				{
					printf("ID FT800 O.K.\n");
					PWRStatus.PWRFT800=1;
				}
				else
					printf("Fault ID FT800\n");
				FT800_SPI_DISABLE();
				//SD on
				printf("SD Power on\n");
				PWR_SD_ON();
				printf("SD Port on\n");
				SD_Port_ON();
				
				FATFS_UnLinkDriver(USERPath);
				FATFS_LinkDriver(&USER_Driver, USERPath);
				
				Init_SD();
				
				//Analog ON
				printf("Analog ON\n");
				PWR_AVBAT_ON();
				PWRStatus.PWRAnalog=1;
				Analog_Port_ON();
				printf("Init AD7794\n");
				HAL_Delay(2);
				ADI_PART_CS_HIGH;
				HAL_Delay(10);
				char error;
				error = AD7794_Init(&hspi1);
				if(error)
					printf("ID AD7794 O.K.\n");
				else
					printf("Fault ID AD7794\n");
				AD7794_SetGain(5);
				AD7794_SetIntReference(0);
				AD7794_SetChannel(0);
				AD7794_SetMode(AD7794_MODE_CONT );
				AD7794_SetRate(Settings.ADCSpeedSetting);
				ADI_PART_CS_LOW; //To sense RDY Pin
				
				//ADC
				MX_ADC1_Init();
				printf("InitADC\n");
				if(HAL_ADC_Start(&hadc1)!=HAL_OK)
					printf("ADC Error\n");

				if(HAL_ADCEx_Calibration_Start(&hadc1)!=HAL_OK)
					printf("ADC_Calibration_Error");

				if(HAL_ADC_Start(&hadc1)!=HAL_OK)
					printf("ADC Error\n");

				HAL_ADC_PollForConversion(&hadc1,1000);
				
				iBatVoltageRaw = HAL_ADC_GetValue(&hadc1);
				fBatVoltage = iBatVoltageRaw * 0.00912f;
				printf("VBat=%6.3f\n",fBatVoltage);
				if(HAL_ADC_Start(&hadc1)!=HAL_OK)
					printf("ADC Error\n");
	
		}

			if(strncmp(CommandBuffer, "setoffset",9) == 0)
			{
					Settings.lOffset = datasum;
					SaveSettings(&Settings);
					printf("Offset Saved\n");
			}
			
			if(strncmp(CommandBuffer, "setkfactor",9) == 0)
			{
					Settings.dFactor = Settings.fCalWeight / (double)(datasum - Settings.lOffset);
					SaveSettings(&Settings);
					printf("Offset Saved\n");
			}

			if(strncmp(CommandBuffer, "setcalweight",12) == 0)
			{
				int result;
				float weight;
				char* ptr = CommandBuffer;
				
				while(*ptr != ' ' && *ptr != 0) 
					ptr++;
				
				result = sscanf(ptr, "%f", &weight);
				if (result == 1)
				{
					Settings.fCalWeight = weight;
					SaveSettings(&Settings);
					printf("CalWeight Saved\n");
				}
			}


			if(strncmp(CommandBuffer, "setsmsnumber",12) == 0)
			{
				char* ptr = CommandBuffer;
				
				while(*ptr != ' ' && *ptr != 0) 
					ptr++;
				while(*ptr == ' ')
					ptr++;
				
				strncpy(Settings.SMSNumber, ptr, sizeof(Settings.SMSNumber));
				SaveSettings(&Settings);
				printf("SMSNumber Saved");
			}
			
			if(strncmp(CommandBuffer, "settime",7) == 0)
			{
				int hour, min, sec, result;
				char* ptr = CommandBuffer;
				
				while(*ptr != ' ' && *ptr != 0) 
					ptr++;
				
				result = sscanf(ptr, "%u %u %u", &hour, &min, &sec);
				if (result == 3)
				{
					rtcTime.hour = hour;
					rtcTime.min = min;
					rtcTime.sec = sec;
					if(LL_RTC_EnterInitMode(RTC)==SUCCESS)
					{
						rtc_settime(&rtcTime);
						LL_RTC_ExitInitMode(RTC);
						printf("Time Set\n");
					}
				}
				else
					printf("eg. settime 10 30 24\n");
			}

			if(strncmp(CommandBuffer, "setalarm",8) == 0)
			{
				uint32_t alarm;
				int result;
				char* ptr = CommandBuffer;
				
				while(*ptr != ' ' && *ptr != 0) 
					ptr++;
				
				result = sscanf(ptr, "%u", &alarm);
				if (result == 1)
				{
					if(LL_RTC_EnterInitMode(RTC)==SUCCESS)
					{
						alarm = alarm + LL_RTC_TIME_Get(RTC);
						LL_RTC_EnableIT_ALR(RTC);
						LL_RTC_ALARM_Set(RTC,alarm);
						LL_RTC_ExitInitMode(RTC);
						printf("Alarm Set\n");
					}
				}
				else
					printf("eg. settime 10 30 24\n");
			}			
				
			
			if(strncmp(CommandBuffer, "getalarm",8) == 0)
			{
					uint32_t test;
					if(LL_RTC_EnterInitMode(RTC)==SUCCESS)
					{
						test = LL_RTC_ALARM_Get(RTC);
						LL_RTC_ExitInitMode(RTC);
						printf("Alarm %x\n",test);
					}
			}

			if(strncmp(CommandBuffer, "setdate",7) == 0)
			{
				int mday, month, year, result;
				char* ptr = CommandBuffer;
				
				while(*ptr != ' ' && *ptr != 0) 
					ptr++;
				
				result = sscanf(ptr, "%u %u %u", &mday, &month, &year);
				if (result == 3)
				{
					rtcTime.mday = mday;
					rtcTime.month = month;
					rtcTime.year = year;
					if(LL_RTC_EnterInitMode(RTC)==SUCCESS)
					{
						rtc_settime(&rtcTime);
						LL_RTC_ExitInitMode(RTC);
						printf("Date Set\n");
					}
				}
				else
					printf("eg. setdate 10 2 2020\n");
			}

			if(strncmp(CommandBuffer, "stop",4) == 0)
			{
				printf("Enter Stop Mode\n");
				LED_Green_OFF();					
				GPIO_InitTypeDef GPIO_InitStruct = {0};
				
				/*Configure GPIO pins as input*/
				GPIO_InitStruct.Pin = GPIO_PIN_All;
				GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
				GPIO_InitStruct.Pull = GPIO_NOPULL;
				HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);					
				HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);					
				HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);					
				HAL_GPIO_Init(GPIOD, &GPIO_InitStruct);

				/*Configure GPIO pin : SYS_WKUP_Pin */
				GPIO_InitStruct.Pin = SYS_WKUP_Pin;
				GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
				GPIO_InitStruct.Pull = GPIO_NOPULL;
				HAL_GPIO_Init(SYS_WKUP_GPIO_Port, &GPIO_InitStruct);					
				
				HAL_PWR_EnterSTOPMode(PWR_LOWPOWERREGULATOR_ON, PWR_SLEEPENTRY_WFI );
				
				SystemClock_Config();
				
				/* Initialize all configured peripherals */

				MX_GPIO_Init();
				
//					MX_ADC1_Init();
				HAL_ADC_MspInit(&hadc1);
//					MX_RTC_Init();
//					MX_SPI1_Init();
				HAL_SPI_MspInit(&hspi1);
//					MX_SPI2_Init();
				HAL_SPI_MspInit(&hspi2);
//					MX_USART1_UART_Init();
				HAL_UART_MspInit(&huart1);
//					MX_USART2_UART_Init();
				HAL_UART_MspInit(&huart2);
//					MX_FATFS_Init();
				
				printf("Resume operation\n");
				LED_Green_ON();
			}

			if(strncmp(CommandBuffer, "stby",4) == 0)
			{
				printf("Enter Stby Mode\n");
				LED_Green_OFF();					
				GPIO_InitTypeDef GPIO_InitStruct = {0};
				
				/*Configure GPIO pins as input*/
				GPIO_InitStruct.Pin = GPIO_PIN_All;
				GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
				GPIO_InitStruct.Pull = GPIO_NOPULL;
				HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);					
				HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);					
				HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);					
				HAL_GPIO_Init(GPIOD, &GPIO_InitStruct);					
				
				HAL_PWR_EnterSTANDBYMode();

			}
			
			if(strcmp(CommandBuffer, "sdtest") == 0)
			{					
				int rc=0;
//					DWORD buff[512];  /* 2048 byte working buffer */
		
				SD_SPI_ENABLE();
					
				//Clock Polarity
				hspi2.Init.CLKPolarity = SPI_POLARITY_HIGH;
				hspi2.Init.CLKPhase = SPI_PHASE_2EDGE;
				hspi2.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_128;
				if (HAL_SPI_Init(&hspi2) != HAL_OK)
				{
					Error_Handler();
				}

				/* Check function/compatibility of the physical drive #0 */
//					rc = test_diskio(0, 1, buff, sizeof buff);
				if (rc) {
						printf("Sorry the function/compatibility test failed.\nFatFs will not work on this disk driver.\n");
				} else {
						printf("Congratulations! The disk I/O layer works well.\n");
				}
				
				SD_SPI_DISABLE();

				//Normal Clock Polarity
				MX_SPI2_Init();
			}
			
			if(strcmp(CommandBuffer, "sdformat") == 0)
			{					

				SD_SPI_ENABLE();
					
				//Clock Polarity
				hspi2.Init.CLKPolarity = SPI_POLARITY_HIGH;
				hspi2.Init.CLKPhase = SPI_PHASE_2EDGE;
				hspi2.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_128;
				if (HAL_SPI_Init(&hspi2) != HAL_OK)
				{
					Error_Handler();
				}
				
				printf("Format SD Card...\n");
				
				if(f_mkfs((TCHAR const*)&USERFatFS, 0, 0) != FR_OK)
				{
					/* FatFs Format Error */
					Error_Handler();
				}
				else
					printf("Format succesful\n");
				
				SD_SPI_DISABLE();

				//Normal Clock Polarity
				MX_SPI2_Init();
			}

			if(strcmp(CommandBuffer, "test") == 0)
			{
				bCommandDone=1;
				printf("Execute Test\n");
			}

			if(strcmp(CommandBuffer, "sim800debug") == 0)
			{
				bCommandDone=1;
				bSIM800Debug = TRUE;
				if(sim800on())
					printf("SIM800 failure\n");
				else
					printf("SIM800 on\n");
			}
			
			if(strcmp(CommandBuffer, "sim800on") == 0)
			{
				bCommandDone=1;
				if(sim800on())
					printf("SIM800 failure\n");
				else
					printf("SIM800 on\n");
			}
			
			if(strcmp(CommandBuffer, "sim800off") == 0)
			{
				bCommandDone=1;
				bSIM800Debug = FALSE;
				
				if(sim800off())
					printf("SIM800 failure\n");
				else
					printf("SIM800 off\n");
			}

			if(strcmp(CommandBuffer, "thingspeak") == 0)
			{
				bCommandDone=1;
				if(sendthingspeak())
					printf("send failure\n");
				else
					printf("send ok\n");
			}

			if(strcmp(CommandBuffer, "testmail") == 0)
			{
				bCommandDone=1;
				if(SendEmail("TEST"))
					printf("send failure\n");
				else
					printf("send ok\n");
			}

			//CommandBuffer an SIM800 senden
			if (bCommandDone==0 && bSIM800Debug)
			{
				strcat(CommandBuffer, "\r\n");
				HAL_UART_Transmit(&huart2, (uint8_t*) CommandBuffer, strlen(CommandBuffer),1000);
			}
		}
		else	//only Echo
			HAL_UART_Transmit(&huart1, (uint8_t*) pCommandBuffer - 1, 1, 1000);
	
		LED_Green_OFF();
	}
	
	if(bCommandOverFlow)
	{
		LED_Green_ON();
		if(HAL_UART_Receive_IT(&huart1, (uint8_t*) pCommandBuffer, 1) == HAL_OK)
				bCommandOverFlow = 0;
		LED_Green_OFF();
	}	
	return 0;
}

void LED_Red_ON(void)
{
	HAL_GPIO_WritePin(LED_RED_GPIO_Port, LED_RED_Pin, GPIO_PIN_RESET);
	return;
}
void LED_Red_OFF(void)
{
	HAL_GPIO_WritePin(LED_RED_GPIO_Port, LED_RED_Pin, GPIO_PIN_SET);
	return;
}
void LED_Green_ON(void)
{
	HAL_GPIO_WritePin(LED_GREEN_GPIO_Port, LED_GREEN_Pin, GPIO_PIN_RESET);
	return;
}
void LED_Green_OFF(void)
{
	HAL_GPIO_WritePin(LED_GREEN_GPIO_Port, LED_GREEN_Pin, GPIO_PIN_SET);
	return;
}
void PWR_3_3VD_ON(void)
{
	HAL_GPIO_WritePin(EN_3_3VD_GPIO_Port, EN_3_3VD_Pin, GPIO_PIN_SET);
	return;
}

void PWR_3_3VD_OFF(void)
{
	HAL_GPIO_WritePin(EN_3_3VD_GPIO_Port, EN_3_3VD_Pin, GPIO_PIN_RESET);
	return;
}
void PWR_AVBAT_ON(void)
{
	HAL_GPIO_WritePin(EN_AVBAT_GPIO_Port, EN_AVBAT_Pin, GPIO_PIN_SET);
	return;
}
void PWR_AVBAT_OFF(void)
{
	HAL_GPIO_WritePin(EN_AVBAT_GPIO_Port, EN_AVBAT_Pin, GPIO_PIN_RESET);
	return;
}
void PWR_4V_ON(void)
{
	HAL_GPIO_WritePin(EN_4V_GPIO_Port, EN_4V_Pin, GPIO_PIN_SET);
	return;
}
void PWR_4V_OFF(void)
{
	HAL_GPIO_WritePin(EN_4V_GPIO_Port, EN_4V_Pin, GPIO_PIN_RESET);
	return;
}

void PWR_FT800_ON(void)
{
	HAL_GPIO_WritePin(FT800_nEN_PWR_GPIO_Port, FT800_nEN_PWR_Pin, GPIO_PIN_RESET);
	return;
}
void PWR_FT800_OFF(void)
{
	HAL_GPIO_WritePin(FT800_nEN_PWR_GPIO_Port, FT800_nEN_PWR_Pin, GPIO_PIN_SET);
	return;
}
void PWR_BACKLIGHT_ON(void)
{
	HAL_GPIO_WritePin(BACKLIGHT_nEN_PWR_GPIO_Port, BACKLIGHT_nEN_PWR_Pin, GPIO_PIN_RESET);
	return;
}
void PWR_BACKLIGHT_OFF(void)
{
	HAL_GPIO_WritePin(BACKLIGHT_nEN_PWR_GPIO_Port, BACKLIGHT_nEN_PWR_Pin, GPIO_PIN_SET);
	return;
}
void PWR_SD_ON(void)
{
	HAL_GPIO_WritePin(SD_nEN_PWR_GPIO_Port, SD_nEN_PWR_Pin, GPIO_PIN_RESET);
	return;
}
void PWR_SD_OFF(void)
{
	HAL_GPIO_WritePin(SD_nEN_PWR_GPIO_Port, SD_nEN_PWR_Pin, GPIO_PIN_SET);
	return;
}

void FT800_SPI_ENABLE(void)
{
	//SD und EEPROM disable
	HAL_GPIO_WritePin(SD_SPI_nEN_GPIO_Port, SD_SPI_nEN_Pin, GPIO_PIN_SET);
	HAL_GPIO_WritePin(EEPROM_nCS_GPIO_Port, EEPROM_nCS_Pin, GPIO_PIN_SET);

	//FT ENABLE
	HAL_GPIO_WritePin(FT800_SPI_nEN_GPIO_Port, FT800_SPI_nEN_Pin, GPIO_PIN_RESET);
	
	return;
}
void FT800_SPI_DISABLE(void)
{
	HAL_GPIO_WritePin(FT800_SPI_nEN_GPIO_Port, FT800_SPI_nEN_Pin, GPIO_PIN_SET);
	return;
}

void EE_CS_ENABLE(void)
{
	//FT800 und SD disable
	HAL_GPIO_WritePin(FT800_SPI_nEN_GPIO_Port, FT800_SPI_nEN_Pin, GPIO_PIN_SET);
	HAL_GPIO_WritePin(SD_SPI_nEN_GPIO_Port, SD_SPI_nEN_Pin, GPIO_PIN_SET);

	//EE enable
	HAL_GPIO_WritePin(EEPROM_nCS_GPIO_Port, EEPROM_nCS_Pin, GPIO_PIN_RESET);
		
	return;
}
void EE_CS_DISABLE(void)
{
	//EE disable
	HAL_GPIO_WritePin(EEPROM_nCS_GPIO_Port, EEPROM_nCS_Pin, GPIO_PIN_SET);
	
	return;
}

void SD_CS_ENABLE(void)
{
	//SD CS enable
	HAL_GPIO_WritePin(SD_nCS_GPIO_Port, SD_nCS_Pin, GPIO_PIN_RESET);
		
	return;
}
void SD_CS_DISABLE(void)
{
	//SD CS disable
	HAL_GPIO_WritePin(SD_nCS_GPIO_Port, SD_nCS_Pin, GPIO_PIN_SET);
	
	return;
}

void SD_SPI_ENABLE(void)
{
	//FT800 und EE disable
	HAL_GPIO_WritePin(FT800_SPI_nEN_GPIO_Port, FT800_SPI_nEN_Pin, GPIO_PIN_SET);
	HAL_GPIO_WritePin(EEPROM_nCS_GPIO_Port, EEPROM_nCS_Pin, GPIO_PIN_SET);

	//SD SPI enable
	HAL_GPIO_WritePin(SD_SPI_nEN_GPIO_Port, SD_SPI_nEN_Pin, GPIO_PIN_RESET);
		
	return;
}
void SD_SPI_DISABLE(void)
{
	//SD disable
	HAL_GPIO_WritePin(SD_SPI_nEN_GPIO_Port, SD_SPI_nEN_Pin, GPIO_PIN_SET);
	
	return;
}

void SD_Port_OFF(void)
{
	SD_SPI_DISABLE();
}

void SD_Port_ON(void)
{
	
}

void Analog_Port_OFF(void)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};
	//Signale SPI1_MOSI(PA7) SPI1_SCK(PA5) AD7794_nCS(PA4)

  GPIO_InitStruct.Pin = SPI1_MOSI_Pin | SPI1_SCK_Pin | AD7794_nCS_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(SPI1_MOSI_GPIO_Port, &GPIO_InitStruct);
}

void Analog_Port_ON(void)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};
	//Signale SPI1_MOSI(PA7) SPI1_SCK(PA5) AD7794_nCS(PA4)

  GPIO_InitStruct.Pin = AD7794_nCS_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
  HAL_GPIO_Init(AD7794_nCS_GPIO_Port, &GPIO_InitStruct);
	
	GPIO_InitStruct.Pin = SPI1_MOSI_Pin | SPI1_SCK_Pin;
	GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
	HAL_GPIO_Init(SPI1_MOSI_GPIO_Port, &GPIO_InitStruct);
}

void Init_SD(void)
{
	SD_SPI_ENABLE();
	
	/* Mount SD Card */
  if(f_mount(&USERFatFS, "", 0) != FR_OK)
    printf("mount failure\n"); //_Error_Handler(__FILE__, __LINE__);
	else
	{
		printf("mount ok\n");
  
		/* Open file to write */
		if(f_open(&USERFile, "first.txt", FA_OPEN_ALWAYS | FA_READ | FA_WRITE) != FR_OK)
			printf("f_open failure\n"); //			_Error_Handler(__FILE__, __LINE__);
		else
		{
			printf("f_open ok\n");

			/* Check free space */
			if(f_getfree("", &fre_clust, &pfs) != FR_OK)
				printf("getfree error\n");//				_Error_Handler(__FILE__, __LINE__);
			else
			{
				total = (uint32_t)((pfs->n_fatent - 2) * pfs->csize * 0.5);
				free = (uint32_t)(fre_clust * pfs->csize * 0.5);
				
				printf("total %ld free %ld\n", total, free);
					
//				/* Free space is less than 1kb */
//				if(free < 1)
//					_Error_Handler(__FILE__, __LINE__);  
//				
//				/* Writing text */
//				f_puts("STM32 SD Card I/O Example via SPI\n", &USERFile);  
//				f_puts("Save the world!!!", &USERFile);
//				
				/* Close file */
				if(f_close(&USERFile) != FR_OK)
					printf("f_close error\n"); //					_Error_Handler(__FILE__, __LINE__);  
				else
				{
					printf("fclose ok\n");
				
// 					/* Open file to read */
// 					if(f_open(&USERFile, "first.txt", FA_READ) != FR_OK)
// 						_Error_Handler(__FILE__, __LINE__);
// 					
// 					while(f_gets(buffer, sizeof(buffer), &USERFile))
// 					{
// 						printf("%s", buffer);
// 					}
// 					
// 					/* Close file */
// 					if(f_close(&USERFile) != FR_OK)
// 						_Error_Handler(__FILE__, __LINE__);     
// 					printf("f_close ok\n");
					/* Unmount SDCARD */
					if(f_mount(NULL, "", 1) != FR_OK)
						printf("unmount error\n"); //						_Error_Handler(__FILE__, __LINE__);  
					else
						printf("unmount ok\n");
				}
			}
		}
	}
	SD_SPI_DISABLE();	
	//Normal Speed
//	MX_SPI2_Init();
}


void SIM800_PWRKEY_LOW(void)
{
	HAL_GPIO_WritePin(SIM800_PWRKEY_GPIO_Port, SIM800_PWRKEY_Pin, GPIO_PIN_RESET);
	return;
}


void SIM800_PWRKEY_HIGH(void)
{
	HAL_GPIO_WritePin(SIM800_PWRKEY_GPIO_Port, SIM800_PWRKEY_Pin, GPIO_PIN_SET);
	return;
}

void SIM800_Port_OFF(void)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};
	//Signale SIM800_DTR(PA1) SIM800_RING(PC3) SIM800_PWRKEY(PC2) SIM800_TX(PA2) SIM800_RX(PA3)

  GPIO_InitStruct.Pin = SIM800_DTR_Pin | SIM800_TX_Pin | SIM800_RX_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(SIM800_DTR_GPIO_Port, &GPIO_InitStruct);
	
  GPIO_InitStruct.Pin = SIM800_RING_Pin | SIM800_PWRKEY_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(SIM800_RING_GPIO_Port, &GPIO_InitStruct);
	
}

void SIM800_Port_ON(void)
{
	GPIO_InitTypeDef GPIO_InitStruct = {0};
	//Signale SIM800_DTR(PA1) SIM800_RING(PC3) SIM800_PWRKEY(PC2) SIM800_TX(PA2) SIM800_RX(PA3)

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOC, SIM800_PWRKEY_Pin, GPIO_PIN_SET);
 
  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(SIM800_DTR_GPIO_Port, SIM800_DTR_Pin, GPIO_PIN_RESET);

  GPIO_InitStruct.Pin = SIM800_PWRKEY_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_OD;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

  GPIO_InitStruct.Pin = SIM800_RING_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(SIM800_RING_GPIO_Port, &GPIO_InitStruct);

  GPIO_InitStruct.Pin = SIM800_DTR_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);
	
	GPIO_InitStruct.Pin = SIM800_TX_Pin;
	GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
	GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
	HAL_GPIO_Init(SIM800_TX_GPIO_Port, &GPIO_InitStruct);

	GPIO_InitStruct.Pin = SIM800_RX_Pin;
	GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(SIM800_RX_GPIO_Port, &GPIO_InitStruct);

}

void FT800_Port_OFF(void)
{
	FT800_SPI_DISABLE();
	HAL_GPIO_WritePin(FT800_nPD_GPIO_Port, FT800_nPD_Pin, GPIO_PIN_RESET);
}

void FT800_Port_ON(void)
{
	
}


void HAL_UART_RxCpltCallback(UART_HandleTypeDef *huart)
{
	
	if(huart == &huart1)
	{
		bCommandBuffer = 1;
		//Received Return?
		if(*pCommandBuffer == 13)
		{
			//Zero at End of Command
			*pCommandBuffer = 0;
			//Set Pointer to Start of Buffer to receive new command
			pCommandBuffer = CommandBuffer;
			bCommandEnter = 1;
		}
		else
			pCommandBuffer++;
		if(!bSDDebug)
			if(HAL_UART_Receive_IT(huart, (uint8_t*)pCommandBuffer, 1) == HAL_BUSY)
				bCommandOverFlow=1;
	
		
		return;
	}

	if(huart == &huart2)
	{ 
		LED_Red_ON();
		bSIM800Buffer = 1;

		if(pSIM800Buffer++ >= (SIM800Buffer + (sizeof(SIM800Buffer)) - 1 ) )
				pSIM800Buffer = SIM800Buffer;

		if(HAL_UART_Receive_IT(huart, (uint8_t*)pSIM800Buffer, 1) == HAL_BUSY)
			bSIM800OverFlow=1;
		LED_Red_OFF();
		return;
	}
}

void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
	if(GPIO_Pin == GPIO_PIN_0) 		// Taster
	{
		bEXTI = 1;
	}
	//else FT800 INT
}


/**
  * @brief  Retargets the C library printf function to the USART.
  * @param  None
  * @retval None
  */
PUTCHAR_PROTOTYPE
{
  /* Place your implementation of fputc here */
  /* e.g. write a character to the USART1 and Loop until the end of transmission */
  HAL_UART_Transmit(&huart1, (uint8_t *)&ch, 1, 0xFFFF);

  return ch;
}

int InitData(structData *pData)
{
	int i;
	
	pData->iNext = 0;
	for(i=0; i<pData->Size; i++)
	{
		pData->fArray[i]=0.0;
	}
	
	return 0;
}

int AddData(structData *pData, float NewValue)
{
	pData->fArray[pData->iNext] = NewValue;
	
	pData->iNext++;
	if(pData->iNext >= pData->Size)
		pData->iNext=0;
	
	return 0;
}

int 	InitSettings(structSettings *pSettings)
{
	int result;
	EE_CS_ENABLE();
	result = EE_Read(ADR_SETTINGS, (char*) pSettings, sizeof(structSettings));
	EE_CS_DISABLE();
	if(result==sizeof(structSettings) )
	{
		printf("Settings read\n");
		return 0;
	}
	else
		return -1;
}

int SaveSettings(structSettings *pSettings)
{
	int result;

	EE_CS_ENABLE();
	result = EE_Write(ADR_SETTINGS, (char*) pSettings, sizeof(structSettings));
	EE_CS_DISABLE();
	if(result==sizeof(structSettings))
	{
		printf("Settings saved\n");
		return 0;
	}
	else
		return -1;
}

int PrintSettings(structSettings *pSettings)
{
	int i;

	for(i=0; i< (sizeof(SettingsIndex) / sizeof(structSettingsIndex)); i++)
	{
		switch(SettingsIndex[i].Settingstype) 
		{
			case Settingsint:
				printf("%u %s = %d\n", i, SettingsIndex[i].szDescription, *(int*)SettingsIndex[i].pSetting);
				break;
			case Settingssz:
				printf("%u %s = %s\n", i, SettingsIndex[i].szDescription, (char*)SettingsIndex[i].pSetting);
				break;
			case Settingslong:
				printf("%u %s = %d\n", i, SettingsIndex[i].szDescription, *(long*)SettingsIndex[i].pSetting);
				break;
			case Settingsulong:
				printf("%u %s = %u\n", i, SettingsIndex[i].szDescription, *(unsigned long*)SettingsIndex[i].pSetting);
				break;
			case Settingsdouble:
				printf("%u %s = %e\n", i, SettingsIndex[i].szDescription, *(double*)SettingsIndex[i].pSetting);
				break;
			case Settingsfloat:
				printf("%u %s = %f\n", i, SettingsIndex[i].szDescription, *(float*)SettingsIndex[i].pSetting);
				break;
			}
	}
	return 0;
}
/*
//PID
int	InitPID(structPID *pPID)
{
	pPID->ulPeriode = Settings.PID.T / 0.001f;
	pPID->ulTick = 0;
	pPID->w = 0.0f;
	pPID->en = 0.0f;
	pPID->un = 0.0f;
	
	return 0;
}

int	PIDSeten(structPID *pPID, float fValue)
{
	pPID->en_1=pPID->en;
	pPID->en = pPID->w - fValue;
	
	pPID->un_1=pPID->un;
	
	pPID->I_1 = pPID->I;
	
	pPID->D_1 = pPID->D;
	
	return 0;
}

int PIDSetw(structPID *pPID, float fValue)
{
	pPID->w = fValue;
	return 0;
}

int CalculatePID(structPID *pPID)
{
	pPID->P =  pPID->en * Settings.PID.kr;
	
	if(pPID->un <= 1.0f && pPID->un >= 0.0f)	//Anti-Windup
		pPID->I = pPID->I_1 + Settings.PID.kr * Settings.PID.T / Settings.PID.tN * pPID->en;
	
	pPID->D = (Settings.PID.tV * Settings.PID.kr *(pPID->en - pPID->en_1 ) + Settings.PID.t1 * (pPID->D_1))/(Settings.PID.t1+Settings.PID.T);
	
	pPID->un = pPID->P + pPID->I + pPID->D;
	
	//printf("w %f en %f P %f I %f D %f un %f\n", pPID->w, pPID->en, pPID->P, pPID->I, pPID->D, pPID->un);
	
	return 0;
}

//PWMOut
int	InitPWMOut(structPWMOut *pPWMOut)
{
	pPWMOut->ulTick = 0;
	pPWMOut->ulPeriode = Settings.PID.tPWM / 0.001f; 
	pPWMOut->ulToggle = 0;
	pPWMOut->fValue = 0.0f;
	return 0;
}
int SetPWMOutValue(structPWMOut *pPWMOut, float fValue)
{
	
	if(Settings.fueOverride >= 0.0f)
		fValue = Settings.fueOverride;
	
	pPWMOut->fValue = fValue;
	if(fValue <= 0.0f)
		pPWMOut->ulToggle = 0;
	else if (fValue >= 1.0f)
		pPWMOut->ulToggle = pPWMOut->ulPeriode;
	else
		pPWMOut->ulToggle = (float)pPWMOut->ulPeriode * fValue;
	return 0;
}

*/
int InitLogtoFile(structLogtoFile *p)
{
	strcpy(p->szFilename, "log.csv");
	
	/* Mount SD Card */
  if(f_mount(&USERFatFS, "", 0) != FR_OK)
			return 1;

	return 0;
}

int LogtoFile(structLogtoFile *p)
{
	uint32_t utc;

	if (!rtc_getutc(&utc)) return 0;
	
	snprintf(buffer,100, "%10lu;%f;%f;%f\n",utc,PWMOut.fValue,ftemp1,ftemp2);
	
	SD_SPI_ENABLE();	
	
	/* Open file to write */
	if(f_open(&USERFile, p->szFilename, FA_OPEN_ALWAYS | FA_READ | FA_WRITE) != FR_OK)
		return 1;
	/* Writing text */
	f_lseek(&USERFile, f_size(&USERFile));
	f_puts(buffer,&USERFile);

	/* Close file */
	if(f_close(&USERFile) != FR_OK)
		return 1;  

	SD_SPI_DISABLE();	
	return 0;
}


int SIM800Buffer_Flush(void)
{
	pSIM800LastCommand = pSIM800Buffer;
	bSIM800Buffer = 0;
	bSIM800OverFlow = 0;
	
	return 0;
}

int StopMode(void)
{	
	//AllOFF
	//SIM800 off
	printf("SIM800 Port off\n");
	SIM800_Port_OFF();
	printf("SIM800 Power off\n");
	PWR_4V_OFF();	
	PWRStatus.PWRSIM800=0;
	//FT800 off
	printf("Backlight off\n");
	PWR_BACKLIGHT_OFF();
	printf("FT800 Port off\n");
	FT800_Port_OFF();
	printf("PWR_FT800_OFF\n");
	PWR_FT800_OFF();
	PWRStatus.PWRFT800=0;
	//SD off
	printf("SD Port off\n");
	SD_Port_OFF();
	printf("SD Power off\n");
	PWR_SD_OFF();

	printf("3.3VD OFF\n");
	PWR_3_3VD_OFF();
	//Analog OFF
	printf("Analog OFF\n");
	Analog_Port_OFF();
	PWRStatus.PWRAnalog=0;
	PWR_AVBAT_OFF();

	printf("DeInit ADC\n");
	HAL_ADC_DeInit(&hadc1);

	printf("Enter Stop Mode\n");
	LED_Green_OFF();					
	GPIO_InitTypeDef GPIO_InitStruct = {0};
	
	/*Configure GPIO pins as input*/
	GPIO_InitStruct.Pin = GPIO_PIN_All;
	GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);					
	HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);					
	HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);					
	HAL_GPIO_Init(GPIOD, &GPIO_InitStruct);

	/*Configure GPIO pin : SYS_WKUP_Pin */
	GPIO_InitStruct.Pin = SYS_WKUP_Pin;
	GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
	GPIO_InitStruct.Pull = GPIO_NOPULL;
	HAL_GPIO_Init(SYS_WKUP_GPIO_Port, &GPIO_InitStruct);					
	
	uint32_t alarm;
	
	//Set time to wakeup
	if(LL_RTC_EnterInitMode(RTC)==SUCCESS)
	{
		alarm = LL_RTC_TIME_Get(RTC) + (Settings.LogToThingSpeakInterval*60);
		LL_RTC_EnableIT_ALR(RTC);
		LL_RTC_ALARM_Set(RTC,alarm);
		LL_RTC_ExitInitMode(RTC);
		printf("Time Set\n");
	}
	
	bEXTI = 0;
	bAD7794ConvReady = bDS18ConvReady = bDHTConvReady = bVBatADCConvReady = 0;
	
	//DEBUG
//	HAL_DBGMCU_EnableDBGStopMode();
//	SysTick->CTRL  = SysTick_CTRL_CLKSOURCE_Msk |
//                   SysTick_CTRL_TICKINT_Msk;	
	
	HAL_PWR_EnterSTOPMode(PWR_LOWPOWERREGULATOR_ON, PWR_SLEEPENTRY_WFI );
	
	//DEBUG
//	SysTick->CTRL  = SysTick_CTRL_CLKSOURCE_Msk |
//                   SysTick_CTRL_TICKINT_Msk   |
//                   SysTick_CTRL_ENABLE_Msk;                         /* Enable SysTick IRQ and SysTick Timer */

	//WAKE UP
	SystemClock_Config();
	
	/* Initialize all configured peripherals */

	MX_GPIO_Init();
	
//					MX_ADC1_Init();
	HAL_ADC_MspInit(&hadc1);
//					MX_RTC_Init();
//					MX_SPI1_Init();
	HAL_SPI_MspInit(&hspi1);
//					MX_SPI2_Init();
	HAL_SPI_MspInit(&hspi2);
//					MX_USART1_UART_Init();
	HAL_UART_MspInit(&huart1);
//					MX_USART2_UART_Init();
	HAL_UART_MspInit(&huart2);
//					MX_FATFS_Init();

	//Debug UART1
	printf("\nBoot...\n");
	
	pCommandBuffer = CommandBuffer;
	bCommandBuffer = 0;
	bCommandOverFlow = 0;

	HAL_UART_Receive_IT(&huart1, (uint8_t*) pCommandBuffer, 1); // Start Receiving on uart1
	
	pSIM800Buffer = SIM800Buffer;
	pSIM800LastCommand = SIM800Buffer;
	bSIM800Buffer = 0;
	bSIM800OverFlow = 0;
	HAL_UART_Receive_IT(&huart2, (uint8_t*) pSIM800Buffer, 1); //Start Receiving on uart2

	if(bEXTI)
	{
		bEXTI=0;
		printf("Resume operation from EXTI\n");
		Mode = ModeActive;
	}
	else
	{
		bEXTI=0;
		printf("Resume operation from RTC\n");
		Mode = ModeLog;
	}
	
	PowerOn();

	return 0;
}

int PowerOn(void)
{
	char error;
	
	LED_Green_ON();

	printf("3.3VD ON\n");
	PWR_3_3VD_ON();

	printf("Init ADC\n");
	HAL_ADC_Init(&hadc1);

	//SIM800 on
	printf("SIM800 Power on\n");
	PWR_4V_ON();					
	printf("SIM800 Port on\n");
	SIM800_Port_ON();
	PWRStatus.PWRSIM800=1;
	
	if(Mode==ModeActive)
	{
		//FT800 on
		printf("PWR_FT800_ON\n");
		PWR_FT800_ON();
		printf("FT800 Port on\n");
		FT800_Port_ON();
		printf("Backlight on\n");
		PWR_BACKLIGHT_ON();
		FT800_SPI_ENABLE();
		printf("Init FT800\n");
		error = FT800_Init(&hspi2);
		if(error)
		{
			printf("ID FT800 O.K.\n");
			PWRStatus.PWRFT800=1;
		}
		else
			printf("Fault ID FT800\n");
	}
	
	FT800_SPI_DISABLE();
	//SD on
	printf("SD Power on\n");
	PWR_SD_ON();
	printf("SD Port on\n");
	SD_Port_ON();

	FATFS_UnLinkDriver(USERPath);
	FATFS_LinkDriver(&USER_Driver, USERPath);

	Init_SD();

	//Analog ON
	printf("Analog ON\n");
	PWR_AVBAT_ON();
	PWRStatus.PWRAnalog=1;
	Analog_Port_ON();
	printf("Init AD7794\n");
	HAL_Delay(2);
	ADI_PART_CS_HIGH;
	HAL_Delay(10);

	error = AD7794_Init(&hspi1);
	if(error)
		printf("ID AD7794 O.K.\n");
	else
		printf("Fault ID AD7794\n");
	AD7794_SetGain(5);
	AD7794_SetIntReference(0);
	AD7794_SetChannel(0);
	AD7794_SetMode(AD7794_MODE_CONT );
	AD7794_SetRate(Settings.ADCSpeedSetting);
	ADI_PART_CS_LOW; //To sense RDY Pin

	//ADC
	MX_ADC1_Init();
	printf("InitADC\n");
	if(HAL_ADC_Start(&hadc1)!=HAL_OK)
		printf("ADC Error\n");

	if(HAL_ADCEx_Calibration_Start(&hadc1)!=HAL_OK)
		printf("ADC_Calibration_Error");

	if(HAL_ADC_Start(&hadc1)!=HAL_OK)
		printf("ADC Error\n");

	HAL_ADC_PollForConversion(&hadc1,1000);

	iBatVoltageRaw = HAL_ADC_GetValue(&hadc1);
	fBatVoltage = iBatVoltageRaw * 0.00912f;
	printf("VBat=%6.3f\n",fBatVoltage);
	if(HAL_ADC_Start(&hadc1)!=HAL_OK)
		printf("ADC Error\n");
	
	if(Settings.LogToThingSpeak)
	{
		MessageBoxText("Start SIM800", 1);
		sim800on();
		MessageBoxText(0,0);
	}
	
	Tick100ms = Tick1s = Tick60s = HAL_GetTick();
	
	return 0;
}

void delayus(int useconds)
{
	volatile int s;
	useconds = useconds *6;
	for (s=0; s<useconds;s++);
}

DWORD AccSize;				/* Work register for fs command */
WORD AccFiles, AccDirs;
FILINFO Finfo;

FATFS FatFs;				/* File system object for each logical drive */
FIL File[2];				/* File objects */
DIR Dir;					/* Directory object */

char Line[256];				/* Console input buffer */
BYTE Buff[4096] __attribute__ ((aligned (4))) ;	/* Working buffer */

volatile UINT Timer;

#define _LINE_ECHO 1

/*----------------------------------------------*/
/* Get a line from the input                    */
/*----------------------------------------------*/

int xgets (		/* 0:End of stream, 1:A line arrived */
	char* buff,	/* Pointer to the buffer */
	int len		/* Buffer length */
)
{
	int i;
	char c;

	i = 0;
	for (;;) {
		while(HAL_UART_Receive(&huart1, (uint8_t*) &c, 1,1000)!=HAL_OK);				/* Get a char from the incoming stream */
		if (!c) return 0;			/* End of stream? */
		if (c == '\r') break;		/* End of line? */
		if (c == '\b' && i) {		/* Back space? */
			i--;
#if _LINE_ECHO
			HAL_UART_Transmit(&huart1, (uint8_t*) &c, 1,1000);
#endif
			continue;
		}
		if (c >= ' ' && i < len - 1) {	/* Visible chars */
			buff[i++] = c;
#if _LINE_ECHO
			HAL_UART_Transmit(&huart1, (uint8_t*) &c, 1,1000);
#endif
		}
	}
	buff[i] = 0;	/* Terminate with a \0 */
#if _LINE_ECHO
#if !_LF_CRLF
		printf("\r");
#endif
		printf("\n");
#endif
	return 1;
}

/*----------------------------------------------*/
/* Dump a line of binary dump                   */
/*----------------------------------------------*/

void put_dump (
	const void* buff,		/* Pointer to the array to be dumped */
	unsigned long addr,		/* Heading address value */
	int len,				/* Number of items to be dumped */
	int width				/* Size of the items (DW_CHAR, DW_SHORT, DW_LONG) */
)
{
	int i;
	const unsigned char *bp;
	const unsigned short *sp;
	const unsigned long *lp;


	printf("%08lX:", addr);		/* address */

	switch (width) {
	case sizeof(char):
		bp = buff;
		for (i = 0; i < len; i++)		/* Hexdecimal dump */
			printf(" %02X", bp[i]);
		printf(" ");
		for (i = 0; i < len; i++)		/* ASCII dump */
			printf("%c",(bp[i] >= ' ' && bp[i] <= '~') ? bp[i] : '.');
		break;
	case sizeof(short):
		sp = buff;
		do								/* Hexdecimal dump */
			printf(" %04X", *sp++);
		while (--len);
		break;
	case sizeof(long):
		lp = buff;
		do								/* Hexdecimal dump */
			printf(" %08LX", *lp++);
		while (--len);
		break;
	}

#if !_LF_CRLF
	printf("\r");
#endif
	printf("\n");
}

/*----------------------------------------------*/
/* Get a value of the string                    */
/*----------------------------------------------*/
/*	"123 -5   0x3ff 0b1111 0377  w "
	    ^                           1st call returns 123 and next ptr
	       ^                        2nd call returns -5 and next ptr
                   ^                3rd call returns 1023 and next ptr
                          ^         4th call returns 15 and next ptr
                               ^    5th call returns 255 and next ptr
                                  ^ 6th call fails and returns 0
*/

int xatoi (			/* 0:Failed, 1:Successful */
	char **str,		/* Pointer to pointer to the string */
	long *res		/* Pointer to the valiable to store the value */
)
{
	unsigned long val;
	unsigned char c, r, s = 0;


	*res = 0;

	while ((c = **str) == ' ') (*str)++;	/* Skip leading spaces */

	if (c == '-') {		/* negative? */
		s = 1;
		c = *(++(*str));
	}

	if (c == '0') {
		c = *(++(*str));
		switch (c) {
		case 'x':		/* hexdecimal */
			r = 16; c = *(++(*str));
			break;
		case 'b':		/* binary */
			r = 2; c = *(++(*str));
			break;
		default:
			if (c <= ' ') return 1;	/* single zero */
			if (c < '0' || c > '9') return 0;	/* invalid char */
			r = 8;		/* octal */
		}
	} else {
		if (c < '0' || c > '9') return 0;	/* EOL or invalid char */
		r = 10;			/* decimal */
	}

	val = 0;
	while (c > ' ') {
		if (c >= 'a') c -= 0x20;
		c -= '0';
		if (c >= 17) {
			c -= 7;
			if (c <= 9) return 0;	/* invalid char */
		}
		if (c >= r) return 0;		/* invalid char for current radix */
		val = val * r + c;
		c = *(++(*str));
	}
	if (s) val = 0 - val;			/* apply sign if needed */

	*res = val;
	return 1;
}

/*--------------------------------------------------------------------------*/
/* Monitor                                                                  */
/*--------------------------------------------------------------------------*/

static
FRESULT scan_files (
	char* path		/* Pointer to the path name working buffer */
)
{
	DIR dirs;
	FRESULT res;
	BYTE i;


	if ((res = f_opendir(&dirs, path)) == FR_OK) {
		i = strlen(path);
		while (((res = f_readdir(&dirs, &Finfo)) == FR_OK) && Finfo.fname[0]) {
			if (Finfo.fattrib & AM_DIR) {
				AccDirs++;
				*(path+i) = '/'; strcpy(path+i+1, Finfo.fname);
				res = scan_files(path);
				*(path+i) = '\0';
				if (res != FR_OK) break;
			} else {
			/*	printf("%s/%s\n", path, fn); */
				AccFiles++;
				AccSize += Finfo.fsize;
			}
		}
	}

	return res;
}



static
void put_rc (int rc)
{
	const char *str =
		"OK\0" "DISK_ERR\0" "INT_ERR\0" "NOT_READY\0" "NO_FILE\0" "NO_PATH\0"
		"INVALID_NAME\0" "DENIED\0" "EXIST\0" "INVALID_OBJECT\0" "WRITE_PROTECTED\0"
		"INVALID_DRIVE\0" "NOT_ENABLED\0" "NO_FILE_SYSTEM\0" "MKFS_ABORTED\0" "TIMEOUT\0"
		"LOCKED\0" "NOT_ENOUGH_CORE\0" "TOO_MANY_OPEN_FILES\0" "INVALID_PARAMETER\0";
	int i;

	for (i = 0; i != rc && *str; i++) {
		while (*str++) ;
	}
	printf("rc=%u FR_%s\n", (UINT)rc, str);
}


static
const char HelpMsg[] =
	"[Disk contorls]\n"
	" di <pd#> - Initialize disk\n"
	" dd [<pd#> <lba>] - Dump a secrtor\n"
	" ds <pd#> - Show disk status\n"
	"[Buffer controls]\n"
	" bd <ofs> - Dump working buffer\n"
	" be <ofs> [<data>] ... - Edit working buffer\n"
	" br <pd#> <lba> [<count>] - Read disk into working buffer\n"
	" bw <pd#> <lba> [<count>] - Write working buffer into disk\n"
	" bf <val> - Fill working buffer\n"
	"[File system controls]\n"
	" fi <ld#> [<mount>]- Force initialized the volume\n"
	" fs [<path>] - Show volume status\n"
	" fl [<path>] - Show a directory\n"
	" fo <mode> <file> - Open a file\n"
	" fc - Close the file\n"
	" fe <ofs> - Move fp in normal seek\n"
	" fd <len> - Read and dump the file\n"
	" fr <len> - Read the file\n"
	" fw <len> <val> - Write to the file\n"
	" fn <org.name> <new.name> - Rename an object\n"
	" fu <name> - Unlink an object\n"
	" fv - Truncate the file at current fp\n"
	" fk <name> - Create a directory\n"
	" fa <atrr> <mask> <object name> - Change attribute of an object\n"
	" ft <year> <month> <day> <hour> <min> <sec> <name> - Change timestamp of an object\n"
	" fx <src.file> <dst.file> - Copy a file\n"
	" fg <path> - Change current directory\n"
	" fq - Show current directory\n"
	" fb <name> - Set volume label\n"
	" fm <ld#> <type> <csize> - Create file system\n"
	" fz [<len>] - Change/Show R/W length for fr/fw/fx command\n"
	"[Misc commands]\n"
	" md[b|h|w] <addr> [<count>] - Dump memory\n"
	" mf <addr> <value> <count> - Fill memory\n"
	" me[b|h|w] <addr> [<value> ...] - Edit memory\n"
	" t [<year> <mon> <mday> <hour> <min> <sec>] - Set/Show RTC\n"
	"\n";


void SDDebug(void)
{
	char *ptr, *ptr2;
	long p1, p2, p3;
	BYTE res, b, drv = 0;
	UINT s1, s2, cnt, blen = sizeof Buff;
	static const char *ft[] = {"", "FAT12", "FAT16", "FAT32", "exFAT"};
	static const char days[] = "Sun\0Mon\0Tue\0Wed\0Thu\0Fri\0Sat";
	DWORD ofs = 0, sect = 0, blk[2];
	FATFS *fs;
	RTCTIME rtc;

	printf("STM32F100 test monitor\n");
	printf(_USE_LFN ? "LFN Enabled" : "LFN Disabled");
	printf(", Code page: %u\n", _CODE_PAGE);

	rtc_gettime(&rtc);
	printf("Current time is %u/%u/%u %2u:%02u:%02u.\n", rtc.year, rtc.month, rtc.mday, rtc.hour, rtc.min, rtc.sec);

	for (;;) 
	{
		printf(">");
		xgets(Line, sizeof Line);

		ptr = Line;
		switch (*ptr++) {
		case '?' :	/* Show Command List */
			printf(HelpMsg);
			break;

		case 'm' :	/* Memory dump/fill/edit */
			switch (*ptr++) {
			case 'd' :	/* md[b|h|w] <address> [<count>] - Dump memory */
				switch (*ptr++) {
				case 'w': p3 = sizeof(long); break;
				case 'h': p3 = sizeof(short); break;
				default: p3 = sizeof(char);
				}
				if (!xatoi(&ptr, &p1)) break;
				if (!xatoi(&ptr, &p2)) p2 = 128 / p3;
				for (ptr = (char*)p1; p2 >= 16 / p3; ptr += 16, p2 -= 16 / p3)
					put_dump(ptr, (DWORD)ptr, 16 / p3, p3);
				if (p2) put_dump((BYTE*)ptr, (UINT)ptr, p2, p3);
				break;
			case 'f' :	/* mf <address> <value> <count> - Fill memory */
				if (!xatoi(&ptr, &p1) || !xatoi(&ptr, &p2) || !xatoi(&ptr, &p3)) break;
				while (p3--) {
					*(BYTE*)p1 = (BYTE)p2;
					p1++;
				}
				break;
			case 'e' :	/* me[b|h|w] <address> [<value> ...] - Edit memory */
				switch (*ptr++) {	/* Get data width */
				case 'w': p3 = sizeof(long); break;
				case 'h': p3 = sizeof(short); break;
				default: p3 = sizeof(char);
				}
				if (!xatoi(&ptr, &p1)) break;	/* Get start address */
				if (xatoi(&ptr, &p2)) {	/* 2nd parameter is given (direct mode) */
					do {
						switch (p3) {
						case sizeof(long): *(DWORD*)p1 = (DWORD)p2; break;
						case sizeof(short): *(WORD*)p1 = (WORD)p2; break;
						default: *(BYTE*)p1 = (BYTE)p2;
						}
						p1 += p3;
					} while (xatoi(&ptr, &p2));	/* Get next value */
					break;
				}
				for (;;) {				/* 2nd parameter is not given (interactive mode) */
					switch (p3) {
					case sizeof(long): printf("%08X 0x%08X-", p1, *(DWORD*)p1); break;
					case sizeof(short): printf("%08X 0x%04X-", p1, *(WORD*)p1); break;
					default: printf("%08X 0x%02X-", p1, *(BYTE*)p1);
					}
//							ptr = Line; xgets(ptr, sizeof Line);
					if (*ptr == '.') break;
					if ((BYTE)*ptr >= ' ') {
						if (!xatoi(&ptr, &p2)) continue;
						switch (p3) {
						case sizeof(long): *(DWORD*)p1 = (DWORD)p2; break;
						case sizeof(short): *(WORD*)p1 = (WORD)p2; break;
						default: *(BYTE*)p1 = (BYTE)p2;
						}
					}
					p1 += p3;
				}
				break;
			}
			break;

		case 'd' :	/* Disk I/O layer controls */
			switch (*ptr++) {
			case 'd' :	/* dd [<pd#> <sect>] - Dump secrtor */
				if (!xatoi(&ptr, &p1)) {
					p1 = drv; p2 = sect;
				} else {
					if (!xatoi(&ptr, &p2)) break;
				}
				drv = (BYTE)p1; sect = p2;
				res = disk_read(drv, Buff, sect, 1);
				if (res) { printf("rc=%d\n", (WORD)res); break; }
				printf("PD#:%u LBA:%lu\n", drv, sect++);
				for (ptr=(char*)Buff, ofs = 0; ofs < 0x200; ptr += 16, ofs += 16)
					put_dump((BYTE*)ptr, ofs, 16, sizeof(char));
				break;

			case 'i' :	/* di <pd#> - Initialize disk */
				if (!xatoi(&ptr, &p1)) break;
			
				FATFS_UnLinkDriver(USERPath);
				FATFS_LinkDriver(&USER_Driver, USERPath);
			
				printf("rc=%d\n", (WORD)disk_initialize((BYTE)p1));
				break;

			case 's' :	/* ds <pd#> - Show disk status */
				if (!xatoi(&ptr, &p1)) break;
				if (disk_ioctl((BYTE)p1, GET_SECTOR_COUNT, &p2) == RES_OK)
					{ printf("Drive size: %lu sectors\n", p2); }
				if (disk_ioctl((BYTE)p1, GET_BLOCK_SIZE, &p2) == RES_OK)
					{ printf("Block size: %lu sectors\n", p2); }
				if (disk_ioctl((BYTE)p1, MMC_GET_TYPE, &b) == RES_OK)
					{ printf("Media type: %u\n", b); }
				if (disk_ioctl((BYTE)p1, MMC_GET_CSD, Buff) == RES_OK)
					{ printf("CSD:\n"); put_dump(Buff, 0, 16, sizeof(char)); }
				if (disk_ioctl((BYTE)p1, MMC_GET_CID, Buff) == RES_OK)
					{ printf("CID:\n"); put_dump(Buff, 0, 16, sizeof(char)); }
				if (disk_ioctl((BYTE)p1, MMC_GET_OCR, Buff) == RES_OK)
					{ printf("OCR:\n"); put_dump(Buff, 0, 4, sizeof(char)); }
				if (disk_ioctl((BYTE)p1, MMC_GET_SDSTAT, Buff) == RES_OK) {
					printf("SD Status:\n");
					for (s1 = 0; s1 < 64; s1 += 16) put_dump(Buff+s1, s1, 16, sizeof(char));
				}
				break;

			case 'c' :	/* Disk ioctl */
				switch (*ptr++) {
				case 's' :	/* dcs <pd#> - CTRL_SYNC */
					if (!xatoi(&ptr, &p1)) break;
					printf("rc=%d\n", disk_ioctl((BYTE)p1, CTRL_SYNC, 0));
					break;
				case 'e' :	/* dce <pd#> <s.lba> <e.lba> - CTRL_TRIM */
					if (!xatoi(&ptr, &p1) || !xatoi(&ptr, (long*)&blk[0]) || !xatoi(&ptr, (long*)&blk[1])) break;
					printf("rc=%d\n", disk_ioctl((BYTE)p1, CTRL_TRIM, blk));
					break;
				}
				break;
			}
			break;

		case 'b' :	/* Buffer controls */
			switch (*ptr++) {
			case 'd' :	/* bd <ofs> - Dump R/W buffer */
				if (!xatoi(&ptr, &p1)) break;
				for (ptr=(char*)&Buff[p1], ofs = p1, cnt = 32; cnt; cnt--, ptr+=16, ofs+=16)
					put_dump((BYTE*)ptr, ofs, 16, sizeof(char));
				break;

			case 'e' :	/* be <ofs> [<data>] ... - Edit R/W buffer */
				if (!xatoi(&ptr, &p1)) break;
				if (xatoi(&ptr, &p2)) {
					do {
						Buff[p1++] = (BYTE)p2;
					} while (xatoi(&ptr, &p2));
					break;
				}
				for (;;) {
					printf("%04X %02X-", (WORD)(p1), (WORD)Buff[p1]);
//							xgets(Line, sizeof Line);
					ptr = Line;
					if (*ptr == '.') break;
					if (*ptr < ' ') { p1++; continue; }
					if (xatoi(&ptr, &p2))
						Buff[p1++] = (BYTE)p2;
					else
						printf("???\n");
				}
				break;

			case 'r' :	/* br <pd#> <lba> [<num>] - Read disk into R/W buffer */
				if (!xatoi(&ptr, &p1) || !xatoi(&ptr, &p2)) break;
				if (!xatoi(&ptr, &p3)) p3 = 1;
				printf("rc=%u\n", (WORD)disk_read((BYTE)p1, Buff, p2, p3));
				break;

			case 'w' :	/* bw <pd#> <lba> [<num>] - Write R/W buffer into disk */
				if (!xatoi(&ptr, &p1) || !xatoi(&ptr, &p2)) break;
				if (!xatoi(&ptr, &p3)) p3 = 1;
				printf("rc=%u\n", (WORD)disk_write((BYTE)p1, Buff, p2, p3));
				break;

			case 'f' :	/* bf <val> - Fill working buffer */
				if (!xatoi(&ptr, &p1)) break;
				memset(Buff, (BYTE)p1, sizeof Buff);
				break;

			}
			break;

		case 'f' :	/* FatFS API controls */
			switch (*ptr++) {

			case 'i' :	/* fi [<opt>]- Initialize logical drive */
				if (!xatoi(&ptr, &p2)) p2 = 0;
				put_rc(f_mount(&FatFs, "", (BYTE)p2));
				break;

			case 's' :	/* fs [<path>] - Show volume status */
				while (*ptr == ' ') ptr++;
				res = f_getfree(ptr, (DWORD*)&p1, &fs);
				if (res) { put_rc(res); break; }
				printf("FAT type = %s\n", ft[fs->fs_type]);
				printf("Bytes/Cluster = %lu\n", (DWORD)fs->csize * 512);
				printf("Number of FATs = %u\n", fs->n_fats);
				if (fs->fs_type < FS_FAT32) printf("Root DIR entries = %u\n", fs->n_rootdir);
				printf("Sectors/FAT = %lu\n", fs->fsize);
				printf("Number of clusters = %lu\n", (DWORD)fs->n_fatent - 2);
				printf("Volume start (lba) = %lu\n", fs->volbase);
				printf("FAT start (lba) = %lu\n", fs->fatbase);
				printf("DIR start (lba,clustor) = %lu\n", fs->dirbase);
				printf("Data start (lba) = %lu\n\n", fs->database);
#if _USE_LABEL
				res = f_getlabel(ptr, (char*)Buff, (DWORD*)&p2);
				if (res) { put_rc(res); break; }
				printf(Buff[0] ? "Volume name is %s\n" : "No volume label\n", (char*)Buff);
				printf("Volume S/N is %04X-%04X\n", (DWORD)p2 >> 16, (DWORD)p2 & 0xFFFF);
#endif
				AccSize = AccFiles = AccDirs = 0;
				printf("...");
				res = scan_files(ptr);
				if (res) { put_rc(res); break; }
				printf("\r%u files, %lu bytes.\n%u folders.\n"
						"%lu KiB total disk space.\n%lu KiB available.\n",
						AccFiles, AccSize, AccDirs,
						(fs->n_fatent - 2) * (fs->csize / 2), (DWORD)p1 * (fs->csize / 2)
				);
				break;

			case 'l' :	/* fl [<path>] - Directory listing */
				while (*ptr == ' ') ptr++;
				res = f_opendir(&Dir, ptr);
				if (res) { put_rc(res); break; }
				p1 = s1 = s2 = 0;
				for(;;) {
					res = f_readdir(&Dir, &Finfo);
					if ((res != FR_OK) || !Finfo.fname[0]) break;
					if (Finfo.fattrib & AM_DIR) {
						s2++;
					} else {
						s1++; p1 += Finfo.fsize;
					}
					printf("%c%c%c%c%c %u/%02u/%02u %02u:%02u %9lu  %s\n",
							(Finfo.fattrib & AM_DIR) ? 'D' : '-',
							(Finfo.fattrib & AM_RDO) ? 'R' : '-',
							(Finfo.fattrib & AM_HID) ? 'H' : '-',
							(Finfo.fattrib & AM_SYS) ? 'S' : '-',
							(Finfo.fattrib & AM_ARC) ? 'A' : '-',
							(Finfo.fdate >> 9) + 1980, (Finfo.fdate >> 5) & 15, Finfo.fdate & 31,
							(Finfo.ftime >> 11), (Finfo.ftime >> 5) & 63,
							Finfo.fsize, Finfo.fname);
				}
				printf("%4u File(s),%10lu bytes total\n%4u Dir(s)", s1, p1, s2);
				res = f_getfree(ptr, (DWORD*)&p1, &fs);
				if (res == FR_OK)
					printf(", %10lu bytes free\n", p1 * fs->csize * 512);
				else
					put_rc(res);
				break;

			case 'o' :	/* fo <mode> <file> - Open a file */
				if (!xatoi(&ptr, &p1)) break;
				while (*ptr == ' ') ptr++;
				put_rc(f_open(&File[0], ptr, (BYTE)p1));
				break;

			case 'c' :	/* fc - Close a file */
				put_rc(f_close(&File[0]));
				break;

			case 'e' :	/* fe - Seek file pointer */
				if (!xatoi(&ptr, &p1)) break;
				res = f_lseek(&File[0], p1);
				put_rc(res);
				if (res == FR_OK)
					printf("fptr=%lu(0x%lX)\n", File[0].fptr, File[0].fptr);
				break;

			case 'd' :	/* fd <len> - read and dump file from current fp */
				if (!xatoi(&ptr, &p1)) break;
				ofs = File[0].fptr;
				while (p1) {
					if ((UINT)p1 >= 16) { cnt = 16; p1 -= 16; }
					else 				{ cnt = p1; p1 = 0; }
					res = f_read(&File[0], Buff, cnt, &cnt);
					if (res != FR_OK) { put_rc(res); break; }
					if (!cnt) break;
					put_dump(Buff, ofs, cnt, sizeof(char));
					ofs += 16;
				}
				break;

			case 'r' :	/* fr <len> - read file */
				if (!xatoi(&ptr, &p1)) break;
				p2 = 0;
				Timer = 0;
				while (p1) {
					if ((UINT)p1 >= blen) {
						cnt = blen; p1 -= blen;
					} else {
						cnt = p1; p1 = 0;
					}
					res = f_read(&File[0], Buff, cnt, &s2);
					if (res != FR_OK) { put_rc(res); break; }
					p2 += s2;
					if (cnt != s2) break;
				}
				printf("%lu bytes read with %lu kB/sec.\n", p2, Timer ? (p2 / Timer) : 0);
				break;

			case 'w' :	/* fw <len> <val> - write file */
				if (!xatoi(&ptr, &p1) || !xatoi(&ptr, &p2)) break;
				memset(Buff, (BYTE)p2, blen);
				p2 = 0;
				Timer = 0;
				while (p1) {
					if ((UINT)p1 >= blen) {
						cnt = blen; p1 -= blen;
					} else {
						cnt = p1; p1 = 0;
					}
					res = f_write(&File[0], Buff, cnt, &s2);
					if (res != FR_OK) { put_rc(res); break; }
					p2 += s2;
					if (cnt != s2) break;
				}
				printf("%lu bytes written with %lu kB/sec.\n", p2, Timer ? (p2 / Timer) : 0);
				break;

			case 'n' :	/* fn <org.name> <new.name> - Change name of an object */
				while (*ptr == ' ') ptr++;
				ptr2 = strchr(ptr, ' ');
				if (!ptr2) break;
				*ptr2++ = 0;
				while (*ptr2 == ' ') ptr2++;
				put_rc(f_rename(ptr, ptr2));
				break;

			case 'u' :	/* fu <name> - Unlink an object */
				while (*ptr == ' ') ptr++;
				put_rc(f_unlink(ptr));
				break;

			case 'v' :	/* fv - Truncate file */
				put_rc(f_truncate(&File[0]));
				break;

			case 'k' :	/* fk <name> - Create a directory */
				while (*ptr == ' ') ptr++;
				put_rc(f_mkdir(ptr));
				break;

			case 'a' :	/* fa <atrr> <mask> <name> - Change attribute of an object */
				if (!xatoi(&ptr, &p1) || !xatoi(&ptr, &p2)) break;
				while (*ptr == ' ') ptr++;
				put_rc(f_chmod(ptr, p1, p2));
				break;

			case 't' :	/* ft <year> <month> <day> <hour> <min> <sec> <name> - Change timestamp of an object */
				if (!xatoi(&ptr, &p1) || !xatoi(&ptr, &p2) || !xatoi(&ptr, &p3)) break;
				Finfo.fdate = ((p1 - 1980) << 9) | ((p2 & 15) << 5) | (p3 & 31);
				if (!xatoi(&ptr, &p1) || !xatoi(&ptr, &p2) || !xatoi(&ptr, &p3)) break;
				Finfo.ftime = ((p1 & 31) << 11) | ((p2 & 63) << 5) | ((p3 >> 1) & 31);
				put_rc(f_utime(ptr, &Finfo));
				break;

			case 'x' : /* fx <src.name> <dst.name> - Copy a file */
				while (*ptr == ' ') ptr++;
				ptr2 = strchr(ptr, ' ');
				if (!ptr2) break;
				*ptr2++ = 0;
				while (*ptr2 == ' ') ptr2++;
				printf("Opening \"%s\"", ptr);
				res = f_open(&File[0], ptr, FA_OPEN_EXISTING | FA_READ);
				printf("\n");
				if (res) {
					put_rc(res);
					break;
				}
				printf("Creating \"%s\"", ptr2);
				res = f_open(&File[1], ptr2, FA_CREATE_ALWAYS | FA_WRITE);
				printf("\n");
				if (res) {
					put_rc(res);
					f_close(&File[0]);
					break;
				}
				printf("Copying file...");
				Timer = 0;
				p1 = 0;
				for (;;) {
					res = f_read(&File[0], Buff, blen, &s1);
					if (res || s1 == 0) break;   /* error or eof */
					res = f_write(&File[1], Buff, s1, &s2);
					p1 += s2;
					if (res || s2 < s1) break;   /* error or disk full */
				}
				printf("\n%lu bytes copied with %lu kB/sec.\n", p1, p1 / Timer);
				f_close(&File[0]);
				f_close(&File[1]);
				break;
#if _FS_RPATH
			case 'g' :	/* fg <path> - Change current directory */
				while (*ptr == ' ') ptr++;
				put_rc(f_chdir(ptr));
				break;
#if _FS_RPATH >= 2
			case 'q' :	/* fq - Show current dir path */
				res = f_getcwd(Line, sizeof Line);
				if (res)
					put_rc(res);
				else
					printf("%s\n", Line);
				break;
#endif
#endif
#if _USE_LABEL
			case 'b' :	/* fb <name> - Set volume label */
				while (*ptr == ' ') ptr++;
				put_rc(f_setlabel(ptr));
				break;
#endif	/* FF_USE_LABEL */
#if _USE_MKFS
			case 'm' :	/* fm [<fs type> [<au size> [<align> [<n_fats> [<n_root>]]]]] - Create filesystem */
				{
					printf("The volume will be formatted. Are you sure? (y/n)=");
					xgets(Line, sizeof Line);
					if (Line[0] == 'y') put_rc(f_mkfs("", 0, 0));
/*
					
					MKFS_PARM opt, *popt = 0;

					if (xatoi(&ptr, &p2)) {
						memset(&opt, 0, sizeof opt);
						popt = &opt;
						popt->fmt = (BYTE)p2;
						if (xatoi(&ptr, &p2)) {
							popt->au_size = p2;
							if (xatoi(&ptr, &p2)) {
								popt->align = p2;
								if (xatoi(&ptr, &p2)) {
									popt->n_fat = (BYTE)p2;
									if (xatoi(&ptr, &p2)) {
										popt->n_root = p2;
									}
								}
							}
						}
					}
					printf("The volume will be formatted. Are you sure? (Y/n)=");
					xgets(Line, sizeof Line);
					if (Line[0] == 'Y') put_rc(f_mkfs("", popt, Buff, sizeof Buff));
*/					
					break;
				}
#endif	/* FF_USE_MKFS */
			case 'z' :	/* fz [<size>] - Change/Show R/W length for fr/fw/fx command */
				if (xatoi(&ptr, &p1) && p1 >= 1 && p1 <= (long)sizeof Buff)
					blen = p1;
				printf("blen=%u\n", blen);
				break;
			}
			break;

		case 't' :	/* t [<year> <mon> <mday> <hour> <min> <sec>] - Set/Show RTC */
			if (xatoi(&ptr, &p1)) {
				rtc.year = (WORD)p1;
				xatoi(&ptr, &p1); rtc.month = (BYTE)p1;
				xatoi(&ptr, &p1); rtc.mday = (BYTE)p1;
				xatoi(&ptr, &p1); rtc.hour = (BYTE)p1;
				xatoi(&ptr, &p1); rtc.min = (BYTE)p1;
				if (!xatoi(&ptr, &p1)) break;
				rtc.sec = (BYTE)p1;
				rtc_settime(&rtc);
			}
			rtc_gettime(&rtc);
			printf("%u/%u/%u %s %02u:%02u:%02u\n", rtc.year, rtc.month, rtc.mday, &days[rtc.wday*4], rtc.hour, rtc.min, rtc.sec);
			break;
		}
	}
}

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  None
  * @retval None
  */
void _Error_Handler(char * file, int line)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  while(1) 
  {
		HAL_NVIC_SystemReset();
  }
  /* USER CODE END Error_Handler_Debug */ 
}

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  while(1) 
  {
		HAL_NVIC_SystemReset();
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{ 
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
	HAL_NVIC_SystemReset();
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
